@@ -134,16 +134,19 @@ enum {
 	TOUCH_UP = 1 << 2,
 	TOUCH_CANCEL = 1 << 3,  // Sent by scrollviews to their children when they detect a scroll
 	TOUCH_WHEEL = 1 << 4,  // Scrollwheel event. Usually only affects Y but can potentially affect X.
-	TOUCH_MOUSE = 1 << 5,  // Identifies that this touch event came from a mouse
+	TOUCH_MOUSE = 1 << 5,  // Identifies that this touch event came from a mouse. Id is now set to the mouse button for DOWN/UP commands.
 	TOUCH_RELEASE_ALL = 1 << 6,  // Useful for app focus switches when events may be lost.
+	TOUCH_HOVER = 1 << 7,
 
-	// These are the Android getToolType() codes, shifted by 10.
+	// These are the Android getToolType() codes, shifted by 10. Unused currently.
 	TOUCH_TOOL_MASK = 7 << 10,
 	TOUCH_TOOL_UNKNOWN = 0 << 10,
 	TOUCH_TOOL_FINGER = 1 << 10,
 	TOUCH_TOOL_STYLUS = 2 << 10,
 	TOUCH_TOOL_MOUSE = 3 << 10,
 	TOUCH_TOOL_ERASER = 4 << 10,
+
+	TOUCH_MAX_POINTERS = 10,
 };
 
 // Used for asynchronous touch input.
@@ -153,6 +156,7 @@ struct TouchInput {
 	float x;
 	float y;
 	int id; // Needs to be <= GestureDetector::MAX_PTRS (10.)
+	int buttons;  // bit mask
 	int flags;
 	double timestamp;
 };
@@ -178,6 +182,11 @@ struct KeyInput {
 		int unicodeChar;  // for KEY_CHAR
 	};
 	int flags;
+
+	// Used by mousewheel events. The delta is packed in the upper 16 bits of flags.
+	int Delta() const {
+		return flags >> 16;
+	}
 };
 
 struct AxisInput {
