@@ -4,8 +4,10 @@
 #include <mutex>
 #include <map>
 #include <functional>
+#include <string_view>
 
 #include "Common/System/System.h"
+#include "Common/File/Path.h"
 
 class Path;
 
@@ -13,6 +15,7 @@ typedef std::function<void(const char *responseString, int responseValue)> Reque
 typedef std::function<void()> RequestFailedCallback;
 
 typedef int RequesterToken;
+
 #define NO_REQUESTER_TOKEN -1
 #define NON_EPHEMERAL_TOKEN -2
 
@@ -31,7 +34,7 @@ public:
 	bool MakeSystemRequest(SystemRequestType type, RequesterToken token, RequestCallback callback, RequestFailedCallback failedCallback, std::string_view param1, std::string_view param2, int64_t param3, int64_t param4 = 0);
 
 	// Called by the platform implementation, when it's finished with a request.
-	void PostSystemSuccess(int requestId, const char *responseString, int responseValue = 0);
+	void PostSystemSuccess(int requestId, std::string_view responseString, int responseValue = 0);
 	void PostSystemFailure(int requestId);
 
 	// This must be called every frame from the beginning of NativeFrame().
@@ -89,9 +92,11 @@ inline void System_InputBoxGetString(RequesterToken token, std::string_view titl
 }
 
 // This one will pop up a special image browser if available. You can also pick
-// images with the file browser below.
-inline void System_BrowseForImage(RequesterToken token, std::string_view title, RequestCallback callback, RequestFailedCallback failedCallback = nullptr) {
-	g_requestManager.MakeSystemRequest(SystemRequestType::BROWSE_FOR_IMAGE, token, callback, failedCallback, title, "", 0);
+// images with the file browser below. If you provide savePath, iOS will be able to
+// convert from HEIC as needed and then save to that path. If this happens, the intParam will
+// be set to 1. Other backends will probably ignore it and set the intParam to 0.
+inline void System_BrowseForImage(RequesterToken token, std::string_view title, Path savePath, RequestCallback callback, RequestFailedCallback failedCallback = nullptr) {
+	g_requestManager.MakeSystemRequest(SystemRequestType::BROWSE_FOR_IMAGE, token, callback, failedCallback, title, savePath.ToString(), 0);
 }
 
 enum class BrowseFileType {
@@ -101,6 +106,9 @@ enum class BrowseFileType {
 	DB,
 	SOUND_EFFECT,
 	ZIP,
+	SYMBOL_MAP,
+	SYMBOL_MAP_NOCASH,
+	ATRAC3,
 	ANY,
 };
 
@@ -108,6 +116,10 @@ inline void System_BrowseForFile(RequesterToken token, std::string_view title, B
 	g_requestManager.MakeSystemRequest(SystemRequestType::BROWSE_FOR_FILE, token, callback, failedCallback, title, "", (int)type);
 }
 
+inline void System_BrowseForFileSave(RequesterToken token, std::string_view title, std::string_view defaultFilename, BrowseFileType type, RequestCallback callback, RequestFailedCallback failedCallback = nullptr) {
+	g_requestManager.MakeSystemRequest(SystemRequestType::BROWSE_FOR_FILE_SAVE, token, callback, failedCallback, title, defaultFilename, (int)type);
+}
+
 void System_BrowseForFolder(RequesterToken token, std::string_view title, const Path &initialPath, RequestCallback callback, RequestFailedCallback failedCallback = nullptr);
 
 // The returned string is username + '\n' + password.
@@ -180,6 +192,15 @@ inline void System_SendDebugScreenshot(std::string_view data, int height) {
 	g_requestManager.MakeSystemRequest(SystemRequestType::SEND_DEBUG_SCREENSHOT, NO_REQUESTER_TOKEN, nullptr, nullptr, data, "", height);
 }
 
+inline void System_IAPRestorePurchases(RequesterToken token, RequestCallback callback, RequestFailedCallback failedCallback = nullptr) {
+	g_requestManager.MakeSystemRequest(SystemRequestType::IAP_RESTORE_PURCHASES, token, callback, failedCallback, "", "", 0);
+}
+
+inline void System_IAPMakePurchase(RequesterToken token, std::string_view productID, RequestCallback callback, RequestFailedCallback failedCallback = nullptr) {
+	g_requestManager.MakeSystemRequest(SystemRequestType::IAP_MAKE_PURCHASE, token, callback, failedCallback, productID, "", 0);
+}
+
+void System_MoveToTrash(const Path &path);
 void System_RunCallbackInWndProc(void (*callback)(void *, void *), void *userdata);
 
 // Non-inline to avoid including Path.h
