@@ -24,12 +24,12 @@
 #include "Common/System/System.h"
 #include "Common/TimeUtil.h"
 #include "Core/Config.h"
-#include "Core/Core.h"
+#include "Core/System.h"
 #include "Core/CoreTiming.h"
 #include "Core/HLE/sceKernel.h"
 #include "Core/HW/Display.h"
 #include "GPU/GPU.h"
-#include "GPU/GPUInterface.h"
+#include "GPU/GPUCommon.h"
 
 // Called when vblank happens (like an internal interrupt.)  Not part of state, should be static.
 static std::mutex listenersLock;
@@ -105,9 +105,15 @@ static void CalculateFPS() {
 
 // TODO: Also average actualFps
 void __DisplayGetFPS(float *out_vps, float *out_fps, float *out_actual_fps) {
-	*out_vps = (float)fps;
-	*out_fps = flips;
-	*out_actual_fps = actualFps;
+	if (out_vps) {
+		*out_vps = (float)fps;
+	}
+	if (out_fps) {
+		*out_fps = flips;
+	}
+	if (out_actual_fps) {
+		*out_actual_fps = actualFps;
+	}
 }
 
 void __DisplayGetVPS(float *out_vps) {
