@@ -16,24 +16,26 @@
 // https://github.com/hrydgard/ppsspp and http://www.ppsspp.org/.
 
 #include <algorithm>
-#include <cstdio>
 
-#include "Common/File/AndroidContentURI.h"
 #include "Common/File/FileUtil.h"
 #include "Common/File/Path.h"
 #include "Common/StringUtils.h"
+#include "Common/Data/Text/I18n.h"
 #include "Core/FileLoaders/CachingFileLoader.h"
 #include "Core/FileLoaders/DiskCachingFileLoader.h"
 #include "Core/FileLoaders/HTTPFileLoader.h"
 #include "Core/FileLoaders/LocalFileLoader.h"
 #include "Core/FileLoaders/RetryingFileLoader.h"
+#include "Core/FileLoaders/ZipFileLoader.h"
 #include "Core/FileSystems/MetaFileSystem.h"
 #include "Core/PSPLoaders.h"
 #include "Core/MemMap.h"
 #include "Core/Loaders.h"
+#include "Core/Core.h"
 #include "Core/System.h"
 #include "Core/ELF/PBPReader.h"
 #include "Core/ELF/ParamSFO.h"
+#include "Core/Util/GameManager.h"
 
 FileLoader *ConstructFileLoader(const Path &filename) {
 	if (filename.Type() == PathType::HTTP) {
@@ -48,6 +50,7 @@ FileLoader *ConstructFileLoader(const Path &filename) {
 }
 
 // TODO : improve, look in the file more
+// Does not take ownership.
 IdentifiedFileType Identify_File(FileLoader *fileLoader, std::string *errorString) {
 	errorString->clear();
 	if (fileLoader == nullptr) {
@@ -64,7 +67,7 @@ IdentifiedFileType Identify_File(FileLoader *fileLoader, std::string *errorStrin
 		return IdentifiedFileType::ERROR_IDENTIFYING;
 	}
 
-	std::string extension = fileLoader->GetPath().GetFileExtension();
+	std::string extension = fileLoader->GetFileExtension();
 	if (extension == ".iso") {
 		// may be a psx iso, they have 2352 byte sectors. You never know what some people try to open
 		if ((fileLoader->FileSize() % 2352) == 0) {
@@ -231,6 +234,29 @@ FileLoader *ResolveFileLoaderTarget(FileLoader *fileLoader) {
 			delete fileLoader;
 			fileLoader = ConstructFileLoader(ebootFilename);
 		}
+	} else if (type == IdentifiedFileType::ARCHIVE_ZIP) {
+		// Handle zip files, take automatic action depending on contents.
+		// Can also return nullptr.
+		ZipFileLoader *zipLoader = new ZipFileLoader(fileLoader);
+
+		ZipFileInfo zipFileInfo{};
+		DetectZipFileContents(zipLoader->GetZip(), &zipFileInfo);
+
+		switch (zipFileInfo.contents) {
+		case ZipFileContents::ISO_FILE:
+		case ZipFileContents::FRAME_DUMP:
+		{
+			zipLoader->Initialize(zipFileInfo.isoFileIndex);
+			return zipLoader;
+		}
+		default:
+		{
+			// Nothing runnable in file. Take the original loader back and return it.
+			fileLoader = zipLoader->Steal();
+			delete zipLoader;
+			return fileLoader;
+		}
+		}
 	}
 	return fileLoader;
 }
@@ -251,133 +277,6 @@ Path ResolvePBPFile(const Path &filename) {
 	}
 }
 
-bool LoadFile(FileLoader **fileLoaderPtr, std::string *error_string) {
-	FileLoader *&fileLoader = *fileLoaderPtr;
-	IdentifiedFileType type = Identify_File(fileLoader, error_string);
-	switch (type) {
-	case IdentifiedFileType::PSP_PBP_DIRECTORY:
-		{
-			fileLoader = ResolveFileLoaderTarget(fileLoader);
-			if (fileLoader->Exists()) {
-				INFO_LOG(Log::Loader, "File is a PBP in a directory: %s", fileLoader->GetPath().c_str());
-				IdentifiedFileType ebootType = Identify_File(fileLoader, error_string);
-				if (ebootType == IdentifiedFileType::PSP_ISO_NP) {
-					InitMemoryForGameISO(fileLoader);
-					pspFileSystem.SetStartingDirectory("disc0:/PSP_GAME/USRDIR");
-					return Load_PSP_ISO(fileLoader, error_string);
-				}
-				else if (ebootType == IdentifiedFileType::PSP_PS1_PBP) {
-					*error_string = "PS1 EBOOTs are not supported by PPSSPP.";
-					coreState = CORE_BOOT_ERROR;
-					return false;
-				} else if (ebootType == IdentifiedFileType::ERROR_IDENTIFYING) {
-					// IdentifyFile will have written to errorString.
-					coreState = CORE_BOOT_ERROR;
-					return false;
-				}
-
-				std::string dir = fileLoader->GetPath().GetDirectory();
-				if (fileLoader->GetPath().Type() == PathType::CONTENT_URI) {
-					dir = AndroidContentURI(dir).FilePath();
-				}
-				size_t pos = dir.find("PSP/GAME/");
-				if (pos != std::string::npos) {
-					dir = ResolvePBPDirectory(Path(dir)).ToString();
-					pspFileSystem.SetStartingDirectory("ms0:/" + dir.substr(pos));
-				}
-				return Load_PSP_ELF_PBP(fileLoader, error_string);
-			} else {
-				*error_string = "No EBOOT.PBP, misidentified game";
-				coreState = CORE_BOOT_ERROR;
-				return false;
-			}
-		}
-		// Looks like a wrong fall through but is not, both paths are handled above.
-
-	case IdentifiedFileType::PSP_PBP:
-	case IdentifiedFileType::PSP_ELF:
-		{
-			INFO_LOG(Log::Loader, "File is an ELF or loose PBP! %s", fileLoader->GetPath().c_str());
-			return Load_PSP_ELF_PBP(fileLoader, error_string);
-		}
-
-	case IdentifiedFileType::PSP_ISO:
-	case IdentifiedFileType::PSP_ISO_NP:
-	case IdentifiedFileType::PSP_DISC_DIRECTORY:	// behaves the same as the mounting is already done by now
-		pspFileSystem.SetStartingDirectory("disc0:/PSP_GAME/USRDIR");
-		return Load_PSP_ISO(fileLoader, error_string);
-
-	case IdentifiedFileType::PSP_PS1_PBP:
-		*error_string = "PS1 EBOOTs are not supported by PPSSPP.";
-		break;
-
-	case IdentifiedFileType::ARCHIVE_RAR:
-#ifdef WIN32
-		*error_string = "RAR file detected (Require WINRAR)";
-#else
-		*error_string = "RAR file detected (Require UnRAR)";
-#endif
-		break;
-
-	case IdentifiedFileType::ARCHIVE_ZIP:
-#ifdef WIN32
-		*error_string = "ZIP file detected (Require WINRAR)";
-#else
-		*error_string = "ZIP file detected (Require UnRAR)";
-#endif
-		break;
-
-	case IdentifiedFileType::ARCHIVE_7Z:
-#ifdef WIN32
-		*error_string = "7z file detected (Require 7-Zip)";
-#else
-		*error_string = "7z file detected (Require 7-Zip)";
-#endif
-		break;
-
-	case IdentifiedFileType::ISO_MODE2:
-		*error_string = "PSX game image detected.";
-		break;
-
-	case IdentifiedFileType::NORMAL_DIRECTORY:
-		ERROR_LOG(Log::Loader, "Just a directory.");
-		*error_string = "Just a directory.";
-		break;
-
-	case IdentifiedFileType::PPSSPP_SAVESTATE:
-		*error_string = "This is a saved state, not a game.";  // Actually, we could make it load it...
-		break;
-
-	case IdentifiedFileType::PSP_SAVEDATA_DIRECTORY:
-		*error_string = "This is save data, not a game."; // Actually, we could make it load it...
-		break;
-
-	case IdentifiedFileType::PPSSPP_GE_DUMP:
-		return Load_PSP_GE_Dump(fileLoader, error_string);
-
-	case IdentifiedFileType::UNKNOWN_BIN:
-	case IdentifiedFileType::UNKNOWN_ELF:
-	case IdentifiedFileType::UNKNOWN_ISO:
-	case IdentifiedFileType::UNKNOWN:
-		ERROR_LOG(Log::Loader, "Unknown file type: %s (%s)", fileLoader->GetPath().c_str(), error_string->c_str());
-		*error_string = "Unknown file type: " + fileLoader->GetPath().ToString();
-		break;
-
-	case IdentifiedFileType::ERROR_IDENTIFYING:
-		*error_string = *error_string + ": " + (fileLoader ? fileLoader->LatestError() : "");
-		ERROR_LOG(Log::Loader, "Error while identifying file: %s", error_string->c_str());
-		break;
-
-	default:
-		*error_string = StringFromFormat("Unhandled identified file type %d", (int)type);
-		ERROR_LOG(Log::Loader, "%s", error_string->c_str());
-		break;
-	}
-
-	coreState = CORE_BOOT_ERROR;
-	return false;
-}
-
 bool UmdReplace(const Path &filepath, FileLoader **fileLoader, std::string &error) {
 	IFileSystem *currentUMD = pspFileSystem.GetSystem("disc0:");
 
@@ -388,8 +287,8 @@ bool UmdReplace(const Path &filepath, FileLoader **fileLoader, std::string &erro
 
 	FileLoader *loadedFile = ConstructFileLoader(filepath);
 
-	if (!loadedFile->Exists()) {
-		error = loadedFile->GetPath().ToVisualString() + " doesn't exist";
+	if (!loadedFile || !loadedFile->Exists()) {
+		error = loadedFile ? (loadedFile->GetPath().ToVisualString() + " doesn't exist") : "no loaded file";
 		delete loadedFile;
 		return false;
 	}
@@ -406,8 +305,8 @@ bool UmdReplace(const Path &filepath, FileLoader **fileLoader, std::string &erro
 	case IdentifiedFileType::PSP_ISO:
 	case IdentifiedFileType::PSP_ISO_NP:
 	case IdentifiedFileType::PSP_DISC_DIRECTORY:
-		if (!ReInitMemoryForGameISO(loadedFile)) {
-			error = "reinit memory failed";
+		if (!MountGameISO(loadedFile)) {
+			error = "mounting the new ISO failed";
 			return false;
 		}
 		break;
@@ -418,3 +317,220 @@ bool UmdReplace(const Path &filepath, FileLoader **fileLoader, std::string &erro
 	}
 	return true;
 }
+
+// Close the return value with ZipClose (if non-null, of course).
+struct zip *ZipOpenPath(const Path &fileName) {
+	int error = 0;
+	// Need to special case for content URI here, similar to OpenCFile.
+	struct zip *z;
+#if PPSSPP_PLATFORM(ANDROID)
+	if (fileName.Type() == PathType::CONTENT_URI) {
+		int fd = File::OpenFD(fileName, File::OPEN_READ);
+		z = zip_fdopen(fd, 0, &error);
+	} else
+#endif
+	{  // continuation of above else in the ifdef
+		z = zip_open(fileName.c_str(), 0, &error);
+	}
+
+	if (!z) {
+		ERROR_LOG(Log::HLE, "Failed to open ZIP file '%s', error code=%i", fileName.c_str(), error);
+	}
+	return z;
+}
+
+void ZipClose(struct zip *z) {
+	if (z)
+		zip_close(z);
+}
+
+bool DetectZipFileContents(const Path &fileName, ZipFileInfo *info) {
+	struct zip *z = ZipOpenPath(fileName);
+	if (!z) {
+		info->contents = ZipFileContents::UNKNOWN;
+		return false;
+	}
+	DetectZipFileContents(z, info);
+	zip_close(z);
+	return true;
+}
+
+static int countSlashes(const std::string &fileName, int *slashLocation) {
+	int slashCount = 0;
+	int lastSlashLocation = -1;
+	if (slashLocation) {
+		*slashLocation = -1;
+	}
+	for (size_t i = 0; i < fileName.size(); i++) {
+		if (fileName[i] == '/') {
+			slashCount++;
+			if (slashLocation) {
+				*slashLocation = lastSlashLocation;
+				lastSlashLocation = (int)i;
+			}
+		}
+	}
+
+	return slashCount;
+}
+
+inline char asciitolower(char in) {
+	if (in <= 'Z' && in >= 'A')
+		return in - ('Z' - 'z');
+	return in;
+}
+
+static bool ZipExtractFileToMemory(struct zip *z, int fileIndex, std::string *data) {
+	struct zip_stat zstat;
+	zip_stat_index(z, fileIndex, 0, &zstat);
+	if (zstat.size == 0) {
+		data->clear();
+		return true;
+	}
+
+	size_t readSize = zstat.size;
+	data->resize(readSize);
+
+	zip_file *zf = zip_fopen_index(z, fileIndex, 0);
+	if (!zf) {
+		ERROR_LOG(Log::HLE, "Failed to zip_fopen_index file %d from zip", fileIndex);
+		return false;
+	}
+
+	zip_int64_t retval = zip_fread(zf, data->data(), readSize);
+	zip_fclose(zf);
+
+	if (retval < 0 || retval < (int)readSize) {
+		ERROR_LOG(Log::HLE, "Failed to read %d bytes from zip (%d) - archive corrupt?", (int)readSize, (int)retval);
+		return false;
+	} else {
+		return true;
+	}
+}
+
+void DetectZipFileContents(struct zip *z, ZipFileInfo *info) {
+	int numFiles = zip_get_num_files(z);
+	_dbg_assert_(numFiles >= 0);
+
+	// Verify that this is a PSP zip file with the correct layout. We also try
+	// to detect simple zipped ISO files, those we'll just "install" to the current
+	// directory of the Games tab (where else?).
+	bool isPSPMemstickGame = false;
+	bool isZippedISO = false;
+	bool isTexturePack = false;
+	bool isFrameDump = false;
+	int stripChars = 0;
+	int isoFileIndex = -1;
+	int stripCharsTexturePack = -1;
+	int textureIniIndex = -1;
+	int filesInRoot = 0;
+	int directoriesInRoot = 0;
+	bool hasParamSFO = false;
+	bool hasIcon0PNG = false;
+	s64 totalFileSize = 0;
+
+	// TODO: It might be cleaner to write separate detection functions, but this big loop doing it all at once
+	// is quite convenient and makes it easy to add shared heuristics.
+	for (int i = 0; i < numFiles; i++) {
+		const char *fn = zip_get_name(z, i, 0);
+
+		zip_stat_t stat{};
+		zip_stat_index(z, i, 0, &stat);
+		totalFileSize += stat.size;
+
+		std::string zippedName = fn;
+		std::transform(zippedName.begin(), zippedName.end(), zippedName.begin(),
+			[](unsigned char c) { return asciitolower(c); });  // Not using std::tolower to avoid Turkish I->ı conversion.
+		// Ignore macos metadata stuff
+		if (startsWith(zippedName, "__macosx/")) {
+			continue;
+		}
+		if (endsWith(zippedName, "/")) {
+			// A directory. Not all zips bother including these.
+			continue;
+		}
+
+		int prevSlashLocation = -1;
+		int slashCount = countSlashes(zippedName, &prevSlashLocation);
+		if (zippedName.find("eboot.pbp") != std::string::npos) {
+			if (slashCount >= 1 && (!isPSPMemstickGame || prevSlashLocation < stripChars + 1)) {
+				stripChars = prevSlashLocation + 1;
+				isPSPMemstickGame = true;
+			} else {
+				INFO_LOG(Log::HLE, "Wrong number of slashes (%i) in '%s'", slashCount, fn);
+			}
+			// TODO: Extract icon and param.sfo from the pbp to be able to display it on the install screen.
+		} else if (endsWith(zippedName, ".iso") || endsWith(zippedName, ".cso") || endsWith(zippedName, ".chd")) {
+			if (slashCount <= 1) {
+				// We only do this if the ISO file is in the root or one level down.
+				isZippedISO = true;
+				INFO_LOG(Log::HLE, "ISO found in zip: %s", zippedName.c_str());
+				if (isoFileIndex != -1) {
+					INFO_LOG(Log::HLE, "More than one ISO file found in zip. Ignoring additional ones.");
+				} else {
+					isoFileIndex = i;
+					info->contentName = zippedName;
+				}
+			}
+		} else if (zippedName.find("textures.ini") != std::string::npos) {
+			int slashLocation = (int)zippedName.find_last_of('/');
+			if (stripCharsTexturePack == -1 || slashLocation < stripCharsTexturePack + 1) {
+				stripCharsTexturePack = slashLocation + 1;
+				isTexturePack = true;
+				textureIniIndex = i;
+			}
+		} else if (endsWith(zippedName, ".ppdmp")) {
+			isFrameDump = true;
+			isoFileIndex = i;
+			info->contentName = zippedName;
+		} else if (endsWith(zippedName, "/param.sfo")) {
+			// Get the game name so we can display it.
+			std::string paramSFOContents;
+			if (ZipExtractFileToMemory(z, i, &paramSFOContents)) {
+				ParamSFOData sfo;
+				if (sfo.ReadSFO((const u8 *)paramSFOContents.data(), paramSFOContents.size())) {
+					if (sfo.HasKey("TITLE")) {
+						info->gameTitle = sfo.GetValueString("TITLE");
+						info->savedataTitle = sfo.GetValueString("SAVEDATA_TITLE");
+						char buff[20];
+						strftime(buff, 20, "%Y-%m-%d %H:%M:%S", localtime(&stat.mtime));
+						info->mTime = buff;
+						info->savedataDetails = sfo.GetValueString("SAVEDATA_DETAIL");
+						info->savedataDir = sfo.GetValueString("SAVEDATA_DIRECTORY");  // should also be parsable from the path.
+						hasParamSFO = true;
+					}
+				}
+			}
+		} else if (endsWith(zippedName, "/icon0.png")) {
+			hasIcon0PNG = true;
+		}
+		if (slashCount == 0) {
+			filesInRoot++;
+		}
+	}
+
+	info->stripChars = stripChars;
+	info->numFiles = numFiles;
+	info->isoFileIndex = isoFileIndex;
+	info->textureIniIndex = textureIniIndex;
+	info->ignoreMetaFiles = false;
+	info->totalFileSize = totalFileSize;
+
+	// Priority ordering for detecting the various kinds of zip file content.s
+	if (isPSPMemstickGame) {
+		info->contents = ZipFileContents::PSP_GAME_DIR;
+	} else if (isZippedISO) {
+		info->contents = ZipFileContents::ISO_FILE;
+	} else if (isTexturePack) {
+		info->stripChars = stripCharsTexturePack;
+		info->ignoreMetaFiles = true;
+		info->contents = ZipFileContents::TEXTURE_PACK;
+	} else if (stripChars == 0 && filesInRoot == 0 && hasParamSFO && hasIcon0PNG) {
+		// As downloaded from GameFAQs, for example.
+		info->contents = ZipFileContents::SAVE_DATA;
+	} else if (isFrameDump) {
+		info->contents = ZipFileContents::FRAME_DUMP;
+	} else {
+		info->contents = ZipFileContents::UNKNOWN;
+	}
+}
