@@ -317,7 +317,7 @@ bool JitBlockCache::RangeMayHaveEmuHacks(u32 start, u32 end) const {
 
 static int binary_search(const JitBlock blocks_[], const u8 *baseoff, int imin, int imax) {
 	while (imin < imax) {
-		int imid = (imin + imax) / 2;
+		int imid = (imin + imax) >> 1;
 		if (blocks_[imid].normalEntry < baseoff)
 			imin = imid + 1;
 		else
