@@ -15,10 +15,7 @@
 // Official git repository and contact information can be found at
 // https://github.com/hrydgard/ppsspp and http://www.ppsspp.org/.
 
-#include <thread>
-
-#include "Common/Data/Encoding/Utf8.h"
-#include "Common/Thread/ThreadUtil.h"
+#include "Core/Core.h"
 #include "Common/System/Request.h"
 
 #include "Common/File/AndroidContentURI.h"
@@ -28,8 +25,8 @@
 #include "Common/CommonWindows.h"
 #endif
 
-#include "Core/ELF/ElfReader.h"
 #include "Core/ELF/ParamSFO.h"
+#include "Core/ELF/PBPReader.h"
 
 #include "Core/FileSystems/BlockDevices.h"
 #include "Core/FileSystems/BlobFileSystem.h"
@@ -42,21 +39,12 @@
 #include "Core/MemMap.h"
 #include "Core/HDRemaster.h"
 
-#include "Core/MIPS/MIPS.h"
-#include "Core/MIPS/MIPSAnalyst.h"
-#include "Core/MIPS/MIPSCodeUtils.h"
 
 #include "Core/Config.h"
 #include "Core/ConfigValues.h"
 #include "Core/System.h"
 #include "Core/PSPLoaders.h"
-#include "Core/HLE/HLE.h"
-#include "Core/HLE/sceKernel.h"
-#include "Core/HLE/sceKernelThread.h"
 #include "Core/HLE/sceKernelModule.h"
-#include "Core/HLE/sceKernelMemory.h"
-
-static std::thread g_loadingThread;
 
 static void UseLargeMem(int memsize) {
 	if (memsize != 1) {
@@ -72,14 +60,7 @@ static void UseLargeMem(int memsize) {
 	}
 }
 
-// We gather the game info before actually loading/booting the ISO
-// to determine if the emulator should enable extra memory and
-// double-sized texture coordinates.
-void InitMemoryForGameISO(FileLoader *fileLoader) {
-	if (!fileLoader->Exists()) {
-		return;
-	}
-
+bool MountGameISO(FileLoader *fileLoader) {
 	std::shared_ptr<IFileSystem> fileSystem;
 	std::shared_ptr<IFileSystem> blockSystem;
 
@@ -88,39 +69,46 @@ void InitMemoryForGameISO(FileLoader *fileLoader) {
 		blockSystem = fileSystem;
 	} else {
 		auto bd = constructBlockDevice(fileLoader);
-		// Can't init anything without a block device...
-		if (!bd)
-			return;
+		if (!bd) {
+			// Can only fail if the ISO is bad.
+			return false;
+		}
 
-		fileSystem = std::make_shared<ISOFileSystem>(&pspFileSystem, bd);
-		blockSystem = std::make_shared<ISOBlockSystem>(fileSystem);
+		auto iso = std::make_shared<ISOFileSystem>(&pspFileSystem, bd);
+		fileSystem = iso;
+		blockSystem = std::make_shared<ISOBlockSystem>(iso);
 	}
 
 	pspFileSystem.Mount("umd0:", blockSystem);
 	pspFileSystem.Mount("umd1:", blockSystem);
-	pspFileSystem.Mount("disc0:", fileSystem);
 	pspFileSystem.Mount("umd:", blockSystem);
-	// TODO: Should we do this?
-	//pspFileSystem.Mount("host0:", fileSystem);
-
-	std::string gameID;
-	std::string umdData;
+	pspFileSystem.Mount("disc0:", fileSystem);
+	return true;
+}
 
+bool LoadParamSFOFromDisc() {
 	std::string sfoPath("disc0:/PSP_GAME/PARAM.SFO");
 	PSPFileInfo fileInfo = pspFileSystem.GetFileInfo(sfoPath.c_str());
-
 	if (fileInfo.exists) {
 		std::vector<u8> paramsfo;
 		pspFileSystem.ReadEntireFile(sfoPath, paramsfo);
 		if (g_paramSFO.ReadSFO(paramsfo)) {
-			UseLargeMem(g_paramSFO.GetValueInt("MEMSIZE"));
-			gameID = g_paramSFO.GetValueString("DISC_ID");
+			return true;
 		}
+	}
+	return false;
+}
 
-		std::vector<u8> umdDataBin;
-		if (pspFileSystem.ReadEntireFile("disc0:/UMD_DATA.BIN", umdDataBin) >= 0) {
-			umdData = std::string((const char *)&umdDataBin[0], umdDataBin.size());
-		}
+// We gather the game info before actually loading/booting the ISO
+// to determine if the emulator should enable extra memory and
+// double-sized texture coordinates.
+void InitMemorySizeForGame() {
+	std::string gameID;
+	std::string umdData;
+
+	if (g_paramSFO.IsValid()) {
+		UseLargeMem(g_paramSFO.GetValueInt("MEMSIZE"));
+		gameID = g_paramSFO.GetValueString("DISC_ID");
 	}
 
 	for (size_t i = 0; i < g_HDRemastersCount; i++) {
@@ -128,6 +116,14 @@ void InitMemoryForGameISO(FileLoader *fileLoader) {
 		if (entry.gameID != gameID) {
 			continue;
 		}
+
+		if (umdData.empty()) {
+			std::vector<u8> umdDataBin;
+			if (pspFileSystem.ReadEntireFile("disc0:/UMD_DATA.BIN", umdDataBin) >= 0) {
+				umdData = std::string((const char *)&umdDataBin[0], umdDataBin.size());
+			}
+		}
+
 		if (entry.umdDataValue && umdData.find(entry.umdDataValue) == umdData.npos) {
 			continue;
 		}
@@ -142,50 +138,14 @@ void InitMemoryForGameISO(FileLoader *fileLoader) {
 	}
 }
 
-bool ReInitMemoryForGameISO(FileLoader *fileLoader) {
-	if (!fileLoader->Exists()) {
-		return false;
-	}
-
-	std::shared_ptr<IFileSystem> fileSystem;
-	std::shared_ptr<IFileSystem> blockSystem;
-
-	if (fileLoader->IsDirectory()) {
-		fileSystem = std::make_shared<VirtualDiscFileSystem>(&pspFileSystem, fileLoader->GetPath());
-		blockSystem = fileSystem;
-	} else {
-		auto bd = constructBlockDevice(fileLoader);
-		if (!bd)
-			return false;
-
-		auto iso = std::make_shared<ISOFileSystem>(&pspFileSystem, bd);
-		fileSystem = iso;
-		blockSystem = std::make_shared<ISOBlockSystem>(iso);
-	}
-
-	pspFileSystem.Remount("umd0:", blockSystem);
-	pspFileSystem.Remount("umd1:", blockSystem);
-	pspFileSystem.Remount("umd:", blockSystem);
-	pspFileSystem.Remount("disc0:", fileSystem);
-
-	return true;
-}
-
-void InitMemoryForGamePBP(FileLoader *fileLoader) {
-	if (!fileLoader->Exists()) {
-		return;
-	}
-
+bool LoadParamSFOFromPBP(FileLoader *fileLoader) {
 	PBPReader pbp(fileLoader);
 	if (pbp.IsValid() && !pbp.IsELF()) {
 		std::vector<u8> sfoData;
 		if (pbp.GetSubFile(PBP_PARAM_SFO, &sfoData)) {
+			// Carefully parse param SFO for PBP files.
 			ParamSFOData paramSFO;
 			if (paramSFO.ReadSFO(sfoData)) {
-				// This is the parameter CFW uses to determine homebrew wants the full 64MB.
-				UseLargeMem(paramSFO.GetValueInt("MEMSIZE"));
-
-				// Take this moment to bring over the title, if set.
 				std::string title = paramSFO.GetValueString("TITLE");
 				if (g_paramSFO.GetValueString("TITLE").empty() && !title.empty()) {
 					g_paramSFO.SetValue("TITLE", title, (int)title.size());
@@ -207,9 +167,11 @@ void InitMemoryForGamePBP(FileLoader *fileLoader) {
 						ver = "1.00";
 					g_paramSFO.SetValue("DISC_VERSION", ver, (int)ver.size());
 				}
+				return true;
 			}
 		}
 	}
+	return false;
 }
 
 
@@ -237,26 +199,14 @@ static const char * const altBootNames[] = {
 };
 
 bool Load_PSP_ISO(FileLoader *fileLoader, std::string *error_string) {
-	// Mounting stuff relocated to InitMemoryForGameISO due to HD Remaster restructuring of code.
-
-	std::string sfoPath("disc0:/PSP_GAME/PARAM.SFO");
-	PSPFileInfo fileInfo = pspFileSystem.GetFileInfo(sfoPath.c_str());
-	if (fileInfo.exists) {
-		std::vector<u8> paramsfo;
-		pspFileSystem.ReadEntireFile(sfoPath, paramsfo);
-		if (g_paramSFO.ReadSFO(paramsfo)) {
-			std::string title = StringFromFormat("%s : %s", g_paramSFO.GetValueString("DISC_ID").c_str(), g_paramSFO.GetValueString("TITLE").c_str());
-			INFO_LOG(Log::Loader, "%s", title.c_str());
-			System_SetWindowTitle(title);
-		}
-	}
-
 	std::string bootpath("disc0:/PSP_GAME/SYSDIR/EBOOT.BIN");
 
 	// Bypass Chinese translation patches, see comment above.
 	for (size_t i = 0; i < ARRAY_SIZE(altBootNames); i++) {
 		if (pspFileSystem.GetFileInfo(altBootNames[i]).exists) {
-			bootpath = altBootNames[i];			
+			WARN_LOG(Log::Boot, "Bypassing suspected translation patch. Booting '%s' instead of '%s'.", altBootNames[i], bootpath.c_str());
+			bootpath = altBootNames[i];
+			// break;  // should have a break here, but it would effectively reverse the evaluation order.
 		}
 	}
 
@@ -271,8 +221,7 @@ bool Load_PSP_ISO(FileLoader *fileLoader, std::string *error_string) {
 
 	bool hasEncrypted = false;
 	int fd;
-	if ((fd = pspFileSystem.OpenFile(bootpath, FILEACCESS_READ)) >= 0)
-	{
+	if ((fd = pspFileSystem.OpenFile(bootpath, FILEACCESS_READ)) >= 0) {
 		u8 head[4];
 		pspFileSystem.ReadFile(fd, head, 4);
 		if (memcmp(head, "~PSP", 4) == 0 || memcmp(head, "\x7F""ELF", 4) == 0) {
@@ -280,6 +229,7 @@ bool Load_PSP_ISO(FileLoader *fileLoader, std::string *error_string) {
 		}
 		pspFileSystem.CloseFile(fd);
 	}
+
 	if (!hasEncrypted) {
 		// try unencrypted Boot.BIN
 		bootpath = "disc0:/PSP_GAME/SYSDIR/BOOT.BIN";
@@ -299,7 +249,6 @@ bool Load_PSP_ISO(FileLoader *fileLoader, std::string *error_string) {
 		} else {
 			*error_string = "A PSP game couldn't be found on the disc.";
 		}
-		coreState = CORE_BOOT_ERROR;
 		return false;
 	}
 
@@ -307,33 +256,11 @@ bool Load_PSP_ISO(FileLoader *fileLoader, std::string *error_string) {
 	g_Config.loadGameConfig(id, g_paramSFO.GetValueString("TITLE"));
 	System_PostUIMessage(UIMessage::CONFIG_LOADED);
 	INFO_LOG(Log::Loader, "Loading %s...", bootpath.c_str());
-
-	PSPLoaders_Shutdown();
-	// Note: this thread reads the game binary, loads caches, and links HLE while UI spins.
-	// To do something deterministically when the game starts, disabling this thread won't be enough.
-	// Instead: Use Core_ListenLifecycle() or watch coreState.
-	g_loadingThread = std::thread([bootpath] {
-		SetCurrentThreadName("ExecLoader");
-		PSP_LoadingLock guard;
-		if (coreState != CORE_POWERUP)
-			return;
-
-		AndroidJNIThreadContext jniContext;
-
-		PSP_SetLoading("Loading executable...");
-		// TODO: We can't use the initial error_string pointer.
-		bool success = __KernelLoadExec(bootpath.c_str(), 0, &PSP_CoreParameter().errorString);
-		if (success && coreState == CORE_POWERUP) {
-			coreState = PSP_CoreParameter().startBreak ? CORE_STEPPING : CORE_RUNNING;
-		} else {
-			coreState = CORE_BOOT_ERROR;
-			// TODO: This is a crummy way to communicate the error...
-			PSP_CoreParameter().fileToStart.clear();
-		}
-	});
-	return true;
+	// TODO: We can't use the initial error_string pointer.
+	return __KernelLoadExec(bootpath.c_str(), 0, &PSP_CoreParameter().errorString);
 }
 
+// TODO: Move this to common. Merge with ResolvePath?
 static Path NormalizePath(const Path &path) {
 	if (path.Type() != PathType::NATIVE) {
 		// Nothing to do - these can't be non-normalized.
@@ -407,7 +334,6 @@ bool Load_PSP_ELF_PBP(FileLoader *fileLoader, std::string *error_string) {
 		// If root is not a subpath of path, we can't boot the game.
 		if (!pathNorm.StartsWith(rootNorm)) {
 			*error_string = "Cannot boot ELF located outside mountRoot.";
-			coreState = CORE_BOOT_ERROR;
 			return false;
 		}
 
@@ -436,25 +362,17 @@ bool Load_PSP_ELF_PBP(FileLoader *fileLoader, std::string *error_string) {
 
 	std::string homebrewName = PSP_CoreParameter().fileToStart.ToVisualString();
 	std::size_t lslash = homebrewName.find_last_of('/');
-#if PPSSPP_PLATFORM(UWP)
-	if (lslash == homebrewName.npos) {
-		lslash = homebrewName.find_last_of("\\");
-	}
-#endif
+	std::size_t rslash = homebrewName.find_last_of('\\');
 	if (lslash != homebrewName.npos)
 		homebrewName = homebrewName.substr(lslash + 1);
-	std::string homebrewTitle = g_paramSFO.GetValueString("TITLE");
-	if (homebrewTitle.empty())
-		homebrewTitle = homebrewName;
+	if (rslash != homebrewName.npos)
+		homebrewName = homebrewName.substr(rslash + 1);
 	std::string discID = g_paramSFO.GetDiscID();
 	std::string discVersion = g_paramSFO.GetValueString("DISC_VERSION");
 	std::string madeUpID = g_paramSFO.GenerateFakeID(Path());
 
-	std::string title = StringFromFormat("%s : %s", discID.c_str(), homebrewTitle.c_str());
-	INFO_LOG(Log::Loader, "%s", title.c_str());
-	System_SetWindowTitle(title);
-
 	// Migrate old save states from old versions of fake game IDs.
+	// Ugh, this might actually be slow on Android.
 	const Path savestateDir = GetSysDirectory(DIRECTORY_SAVESTATE);
 	for (int i = 0; i < 5; ++i) {
 		Path newPrefix = savestateDir / StringFromFormat("%s_%s_%d", discID.c_str(), discVersion.c_str(), i);
@@ -471,55 +389,12 @@ bool Load_PSP_ELF_PBP(FileLoader *fileLoader, std::string *error_string) {
 			File::Rename(oldNamePrefix.WithExtraExtension(".jpg"), newPrefix.WithExtraExtension(".jpg"));
 	}
 
-	PSPLoaders_Shutdown();
-	// Note: See Load_PSP_ISO for notes about this thread.
-	g_loadingThread = std::thread([finalName] {
-		SetCurrentThreadName("ExecLoader");
-		PSP_LoadingLock guard;
-		if (coreState != CORE_POWERUP)
-			return;
-
-		AndroidJNIThreadContext jniContext;
-
-		bool success = __KernelLoadExec(finalName.c_str(), 0, &PSP_CoreParameter().errorString);
-		if (success && coreState == CORE_POWERUP) {
-			coreState = PSP_CoreParameter().startBreak ? CORE_STEPPING : CORE_RUNNING;
-		} else {
-			coreState = CORE_BOOT_ERROR;
-			// TODO: This is a crummy way to communicate the error...
-			PSP_CoreParameter().fileToStart.clear();
-		}
-	});
-	return true;
+	return __KernelLoadExec(finalName.c_str(), 0, error_string);
 }
 
 bool Load_PSP_GE_Dump(FileLoader *fileLoader, std::string *error_string) {
 	auto umd = std::make_shared<BlobFileSystem>(&pspFileSystem, fileLoader, "data.ppdmp");
 	pspFileSystem.Mount("disc0:", umd);
 
-	PSPLoaders_Shutdown();
-	// Note: See Load_PSP_ISO for notes about this thread.
-	g_loadingThread = std::thread([] {
-		SetCurrentThreadName("ExecLoader");
-		PSP_LoadingLock guard;
-		if (coreState != CORE_POWERUP)
-			return;
-
-		AndroidJNIThreadContext jniContext;
-
-		bool success = __KernelLoadGEDump("disc0:/data.ppdmp", &PSP_CoreParameter().errorString);
-		if (success && coreState == CORE_POWERUP) {
-			coreState = PSP_CoreParameter().startBreak ? CORE_STEPPING : CORE_RUNNING;
-		} else {
-			coreState = CORE_BOOT_ERROR;
-			// TODO: This is a crummy way to communicate the error...
-			PSP_CoreParameter().fileToStart.clear();
-		}
-	});
-	return true;
-}
-
-void PSPLoaders_Shutdown() {
-	if (g_loadingThread.joinable())
-		g_loadingThread.join();
+	return __KernelLoadGEDump("disc0:/data.ppdmp", &PSP_CoreParameter().errorString);
 }
