@@ -24,7 +24,10 @@ class FileLoader;
 bool Load_PSP_ISO(FileLoader *fileLoader, std::string *error_string);
 bool Load_PSP_ELF_PBP(FileLoader *fileLoader, std::string *error_string);
 bool Load_PSP_GE_Dump(FileLoader *fileLoader, std::string *error_string);
-void InitMemoryForGameISO(FileLoader *fileLoader);
-bool ReInitMemoryForGameISO(FileLoader *fileLoader);
-void InitMemoryForGamePBP(FileLoader *fileLoader);
+
+bool MountGameISO(FileLoader *fileLoader);
+bool LoadParamSFOFromDisc();
+bool LoadParamSFOFromPBP(FileLoader *fileLoader);
+void InitMemorySizeForGame();
+
 void PSPLoaders_Shutdown();
