@@ -13,15 +13,9 @@
 // hash = md5_finalize()
 
 #include <algorithm>
-#include <atomic>
-#include <cstdarg>
-#include <cstdlib>
-#include <ctime>
-#include <functional>
 #include <set>
 #include <string>
 #include <vector>
-#include <mutex>
 
 #include "ext/rcheevos/include/rcheevos.h"
 #include "ext/rcheevos/include/rc_client.h"
@@ -38,8 +32,7 @@
 #include "Common/Crypto/md5.h"
 #include "Common/Log.h"
 #include "Common/File/Path.h"
-#include "Common/File/FileUtil.h"
-#include "Common/Net/HTTPClient.h"
+#include "Common/Net/HTTPRequest.h"
 #include "Common/System/OSD.h"
 #include "Common/System/System.h"
 #include "Common/System/NativeApp.h"
@@ -48,18 +41,14 @@
 #include "Common/Serialize/Serializer.h"
 #include "Common/Serialize/SerializeFuncs.h"
 #include "Common/StringUtils.h"
-#include "Common/Crypto/md5.h"
 #include "Common/UI/IconCache.h"
+#include "Core/ELF/ParamSFO.h"
 
 #include "Core/MemMap.h"
 #include "Core/Config.h"
-#include "Core/CoreParameter.h"
 #include "Core/Core.h"
 #include "Core/System.h"
-#include "Core/FileLoaders/LocalFileLoader.h"
 #include "Core/FileSystems/BlockDevices.h"
-#include "Core/ELF/ParamSFO.h"
-#include "Core/FileSystems/MetaFileSystem.h"
 #include "Core/FileSystems/ISOFileSystem.h"
 #include "Core/RetroAchievements.h"
 
@@ -69,6 +58,8 @@
 
 #endif
 
+static const char *const RAINTEGRATION_FILENAME = "RAIntegration.dll";
+
 static bool HashISOFile(ISOFileSystem *fs, const std::string filename, md5_context *md5) {
 	int handle = fs->OpenFile(filename, FILEACCESS_READ);
 	if (handle < 0) {
@@ -274,7 +265,7 @@ static uint32_t read_memory_callback(uint32_t address, uint8_t *buffer, uint32_t
 	uint32_t orig_address = address;
 	address += PSP_MEMORY_OFFSET;
 
-	if (!Memory::ValidSize(address, num_bytes)) {
+	if (!Memory::IsValidRange(address, num_bytes)) {
 		// Some achievement packs are really, really spammy.
 		// So we'll just count the bad accesses.
 		Achievements::g_stats.badMemoryAccessCount++;
@@ -298,7 +289,7 @@ static void server_call_callback(const rc_api_request_t *request,
 	// If post data is provided, we need to make a POST request, otherwise, a GET request will suffice.
 	auto ac = GetI18NCategory(I18NCat::ACHIEVEMENTS);
 	if (request->post_data) {
-		std::shared_ptr<http::Request> download = g_DownloadManager.AsyncPostWithCallback(std::string(request->url), std::string(request->post_data), "application/x-www-form-urlencoded", http::ProgressBarMode::DELAYED, [=](http::Request &download) {
+		std::shared_ptr<http::Request> download = g_DownloadManager.AsyncPostWithCallback(std::string(request->url), std::string(request->post_data), "application/x-www-form-urlencoded", http::RequestFlags::ProgressBar | http::RequestFlags::ProgressBarDelayed, [=](http::Request &download) {
 			std::string buffer;
 			download.buffer().TakeAll(&buffer);
 			rc_api_server_response_t response{};
@@ -308,7 +299,7 @@ static void server_call_callback(const rc_api_request_t *request,
 			callback(&response, callback_data);
 		}, ac->T("Contacting RetroAchievements server..."));
 	} else {
-		std::shared_ptr<http::Request> download = g_DownloadManager.StartDownloadWithCallback(std::string(request->url), Path(), http::ProgressBarMode::DELAYED, [=](http::Request &download) {
+		std::shared_ptr<http::Request> download = g_DownloadManager.StartDownloadWithCallback(std::string(request->url), Path(), http::RequestFlags::ProgressBar | http::RequestFlags::ProgressBarDelayed, [=](http::Request &download) {
 			std::string buffer;
 			download.buffer().TakeAll(&buffer);
 			rc_api_server_response_t response{};
@@ -554,7 +545,7 @@ static void raintegration_event_handler(const rc_client_raintegration_event_t *e
 		break;
 	case RC_CLIENT_RAINTEGRATION_EVENT_PAUSE:
 		// The toolkit has hit a breakpoint and wants to pause the emulator. Do so.
-		Core_EnableStepping(true, "ra_breakpoint");
+		Core_Break(BreakReason::RABreak);
 		break;
 	case RC_CLIENT_RAINTEGRATION_EVENT_HARDCORE_CHANGED:
 		// Hardcore mode has been changed (either directly by the user, or disabled through the use of the tools).
@@ -563,7 +554,7 @@ static void raintegration_event_handler(const rc_client_raintegration_event_t *e
 		g_Config.bAchievementsHardcoreMode = rc_client_get_hardcore_enabled(client);
 		break;
 	default:
-		ERROR_LOG(Log::Achievements, "Unsupported raintegration event %u\n", event->type);
+		ERROR_LOG(Log::Achievements, "Unsupported RAIntegration event %u\n", event->type);
 		break;
 	}
 }
@@ -576,7 +567,7 @@ static void load_integration_callback(int result, const char *error_message, rc_
 	case RC_OK:
 	{
 		// DLL was loaded correctly.
-		g_OSD.Show(OSDType::MESSAGE_SUCCESS, ac->T("RAIntegration DLL loaded."));
+		g_OSD.Show(OSDType::MESSAGE_SUCCESS, ApplySafeSubstitutions(ac->T("%1 loaded."), RAINTEGRATION_FILENAME));
 
 		rc_client_raintegration_set_console_id(g_rcClient, RC_CONSOLE_PSP);
 		rc_client_raintegration_set_event_handler(g_rcClient, &raintegration_event_handler);
@@ -591,11 +582,11 @@ static void load_integration_callback(int result, const char *error_message, rc_
 	}
 	case RC_MISSING_VALUE:
 		// This is fine, proceeding to login.
-		g_OSD.Show(OSDType::MESSAGE_WARNING, ac->T("RAIntegration is enabled, but RAIntegration-x64.dll was not found."));
+		g_OSD.Show(OSDType::MESSAGE_WARNING, ac->T("RAIntegration is enabled, but %1 was not found."));
 		break;
 	case RC_ABORTED:
-		// This is fine, proceeding to login.
-		g_OSD.Show(OSDType::MESSAGE_WARNING, ac->T("Wrong version of RAIntegration-x64.dll?"));
+		// This is fine(-ish), proceeding to login.
+		g_OSD.Show(OSDType::MESSAGE_WARNING, ApplySafeSubstitutions("Wrong version of %1?", RAINTEGRATION_FILENAME));
 		break;
 	default:
 		g_OSD.Show(OSDType::MESSAGE_ERROR, StringFromFormat("RAIntegration init failed: %s", error_message));
@@ -755,16 +746,20 @@ void UpdateSettings() {
 
 bool Shutdown() {
 	g_activeChallenges.clear();
+	if (g_rcClient) {
 #ifdef RC_CLIENT_SUPPORTS_RAINTEGRATION
-	rc_client_unload_raintegration(g_rcClient);
+		rc_client_unload_raintegration(g_rcClient);
 #endif
-	rc_client_destroy(g_rcClient);
-	g_rcClient = nullptr;
-	INFO_LOG(Log::Achievements, "Achievements shut down.");
+		rc_client_destroy(g_rcClient);
+		g_rcClient = nullptr;
+		INFO_LOG(Log::Achievements, "Achievements shut down.");
+	}
 	return true;
 }
 
 void ResetRuntime() {
+	if (!g_rcClient)
+		return;
 	INFO_LOG(Log::Achievements, "Resetting rcheevos state...");
 	rc_client_reset(g_rcClient);
 	g_activeChallenges.clear();
@@ -883,7 +878,7 @@ bool HasAchievementsOrLeaderboards() {
 void DownloadImageIfMissing(const std::string &cache_key, std::string &&url) {
 	if (g_iconCache.MarkPending(cache_key)) {
 		INFO_LOG(Log::Achievements, "Downloading image: %s (%s)", url.c_str(), cache_key.c_str());
-		g_DownloadManager.StartDownloadWithCallback(url, Path(), http::ProgressBarMode::NONE, [cache_key](http::Request &download) {
+		g_DownloadManager.StartDownloadWithCallback(url, Path(), http::RequestFlags::Default, [cache_key](http::Request &download) {
 			if (download.ResultCode() != 200)
 				return;
 			std::string data;
@@ -950,13 +945,34 @@ void identify_and_load_callback(int result, const char *error_message, rc_client
 		if (RC_OK == rc_client_game_get_image_url(gameInfo, temp, sizeof(temp))) {
 			Achievements::DownloadImageIfMissing(cacheId, std::string(temp));
 		}
-		g_OSD.Show(OSDType::MESSAGE_INFO, std::string(gameInfo->title), GetGameAchievementSummary(), cacheId, 5.0f);
+
+		GameRegion region = DetectGameRegionFromID(g_paramSFO.GetDiscID());
+		auto ga = GetI18NCategory(I18NCat::GAME);
+		std::string_view regionStr = ga->T(GameRegionToString(region));
+		std::string title(gameInfo->title);
+		if (region != GameRegion::OTHER) {
+			title += " (";
+			title += regionStr;
+			title += ")";
+		}
+		g_OSD.Show(OSDType::MESSAGE_INFO, title, GetGameAchievementSummary(), cacheId, 5.0f);
 		break;
 	}
 	case RC_NO_GAME_LOADED:
+	{
+		GameRegion region = DetectGameRegionFromID(g_paramSFO.GetDiscID());
+		auto ga = GetI18NCategory(I18NCat::GAME);
+		std::string_view regionStr = ga->T(GameRegionToString(region));
+		std::string title(g_paramSFO.GetValueString("TITLE"));
+		if (region != GameRegion::OTHER) {
+			title += " (";
+			title += regionStr;
+			title += ")";
+		}
 		// The current game does not support achievements.
-		g_OSD.Show(OSDType::MESSAGE_INFO, ac->T("RetroAchievements are not available for this game"), "", g_RAImageID, 3.0f);
+		g_OSD.Show(OSDType::MESSAGE_INFO, title, ac->T("RetroAchievements are not available for this game"), g_RAImageID, 3.0f);
 		break;
+	}
 	case RC_NO_RESPONSE:
 		// We lost the internet connection at some point and can't report achievements.
 		ShowNotLoggedInMessage();
@@ -971,10 +987,6 @@ void identify_and_load_callback(int result, const char *error_message, rc_client
 	g_isIdentifying = false;
 }
 
-bool IsReadyToStart() {
-	return !g_isLoggingIn;
-}
-
 void SetGame(const Path &path, IdentifiedFileType fileType, FileLoader *fileLoader) {
 	bool homebrew = false;
 	switch (fileType) {
@@ -1008,6 +1020,11 @@ void SetGame(const Path &path, IdentifiedFileType fileType, FileLoader *fileLoad
 		return;
 	}
 
+	if (!fileLoader) {
+		ERROR_LOG(Log::Achievements, "File loader not initialized");
+		return;
+	}
+
 	// The caller should hold off on executing game code until this turns false, checking with IsBlockingExecution()
 	g_gamePath = path;
 	g_isIdentifying = true;
