@@ -25,19 +25,18 @@
 // All credit goes to him and the official miniupnp project! http://miniupnp.free.fr/
 
 
-#include <algorithm>
-#include <cstdlib>
+#include <algorithm>  // find_if
 #include <cstring>
 #include <string>
 #include <thread>
 
 #include "Common/TimeUtil.h"
 #include "Common/Data/Text/I18n.h"
-#include "Common/Net/Resolve.h"
 #include "Common/Thread/ThreadUtil.h"
 #include "Common/System/OSD.h"
 #include "Common/Log.h"
 #include "Core/Config.h"
+#include "Core/Core.h"
 #include "Core/System.h"
 #include "Core/ELF/ParamSFO.h"
 #include "Core/Util/PortManager.h"
@@ -470,15 +469,14 @@ bool PortManager::RefreshPortList() {
 #endif // WITH_UPNP
 }
 
-int upnpService(const unsigned int timeout)
-{
+int upnpService(const unsigned int timeout) {
 	SetCurrentThreadName("UPnPService");
 	INFO_LOG(Log::sceNet, "UPnPService: Begin of UPnPService Thread");
 
 	// Service Loop
-	while (upnpServiceRunning && coreState != CORE_POWERDOWN) {
+	while (upnpServiceRunning) {
 		// Sleep for 1ms for faster response if active, otherwise sleep longer (TODO: Improve on this).
-		sleep_ms(g_Config.bEnableUPnP ? 1 : 100);
+		sleep_ms(g_Config.bEnableUPnP ? 1 : 100, "upnp-poll");
 
 		// Attempts to reconnect if not connected yet or got disconnected
 		if (g_Config.bEnableUPnP && g_PortManager.GetInitState() == UPNP_INITSTATE_NONE) {
@@ -525,10 +523,10 @@ int upnpService(const unsigned int timeout)
 	return 0;
 }
 
-void __UPnPInit(const unsigned int timeout) {
+void __UPnPInit(const int timeout_ms) {
 	if (!upnpServiceRunning) {
 		upnpServiceRunning = true;
-		upnpServiceThread = std::thread(upnpService, timeout);
+		upnpServiceThread = std::thread(upnpService, timeout_ms);
 	}
 }
 
