@@ -17,7 +17,6 @@
 
 #include "ppsspp_config.h"
 
-#include <algorithm>
 #include <cstring>
 #include <memory>
 #include <png.h>
@@ -25,26 +24,19 @@
 #include "ext/basis_universal/basisu_transcoder.h"
 #include "ext/xxhash.h"
 
-#include "Common/Data/Convert/ColorConv.h"
 #include "Common/Data/Format/IniFile.h"
-#include "Common/Data/Format/ZIMLoad.h"
-#include "Common/Data/Format/PNGLoad.h"
 #include "Common/Data/Text/I18n.h"
 #include "Common/Data/Text/Parsers.h"
 #include "Common/File/VFS/DirectoryReader.h"
 #include "Common/File/VFS/ZipFileReader.h"
 #include "Common/File/FileUtil.h"
 #include "Common/File/VFS/VFS.h"
-#include "Common/LogReporting.h"
 #include "Common/StringUtils.h"
 #include "Common/System/OSD.h"
-#include "Common/Thread/ParallelLoop.h"
-#include "Common/Thread/Waitable.h"
 #include "Common/Thread/ThreadManager.h"
 #include "Common/TimeUtil.h"
 #include "Core/Config.h"
 #include "Core/System.h"
-#include "Core/ThreadPools.h"
 #include "Core/ELF/ParamSFO.h"
 #include "GPU/Common/TextureReplacer.h"
 #include "GPU/Common/TextureDecoder.h"
@@ -88,32 +80,50 @@ void TextureReplacer::NotifyConfigChanged() {
 
 		// If we're saving, auto-create the directory.
 		if (saveEnabled_ && !File::Exists(newTextureDir_)) {
-			INFO_LOG(Log::G3D, "Creating new texture directory: '%s'", newTextureDir_.ToVisualString().c_str());
+			INFO_LOG(Log::TexReplacement, "Creating new texture directory: '%s'", newTextureDir_.ToVisualString().c_str());
 			File::CreateFullPath(newTextureDir_);
 			// We no longer create a nomedia file here, since we put one
 			// in the TEXTURES root.
 		}
 	}
 
+	if (!replaceEnabled_ && wasReplaceEnabled) {
+		delete vfs_;
+		vfs_ = nullptr;
+		Decimate(ReplacerDecimateMode::ALL);
+	} else if (!wasReplaceEnabled && replaceEnabled_) {
+		std::string error;
+		replaceEnabled_ = LoadIni(&error);
+		if (!error.empty() && !replaceEnabled_) {
+			ERROR_LOG(Log::G3D, "ERROR: %s", error.c_str());
+			g_OSD.Show(OSDType::MESSAGE_ERROR, error, 5.0f);
+		}
+	} else if (saveEnabled_) {
+		// Even if just saving is enabled, it makes sense to reload the ini to get the correct
+		// settings for saving. See issue #19086. This can be expensive though.
+		std::string error;
+		bool result = LoadIni(&error);
+		if (!result) {
+			// Ignore errors here, just log if we successfully loaded an ini.
+		} else {
+			INFO_LOG(Log::G3D, "Loaded INI file for saving.");
+		}
+	}
+
 	if (saveEnabled_) {
 		// Somewhat crude message, re-using translation strings.
 		auto d = GetI18NCategory(I18NCat::DEVELOPER);
 		auto di = GetI18NCategory(I18NCat::DIALOG);
-		g_OSD.Show(OSDType::MESSAGE_INFO, std::string(d->T("Save new textures")) + ": " + std::string(di->T("Enabled")), 2.0f);
-	}
-
-	if (!replaceEnabled_ && wasReplaceEnabled) {
-		delete vfs_;
-		vfs_ = nullptr;
-		Decimate(ReplacerDecimateMode::ALL);
-	}
-
-	if (replaceEnabled_) {
-		replaceEnabled_ = LoadIni();
+		std::string str(d->T("Save new textures"));
+		if (!str.empty()) {
+			str.append(": ");
+			str.append(di->T("Enabled"));
+			g_OSD.Show(OSDType::MESSAGE_INFO, str, 2.0f);
+		}
 	}
 }
 
-bool TextureReplacer::LoadIni() {
+bool TextureReplacer::LoadIni(std::string *error) {
 	hash_ = ReplacedTextureHash::QUICK;
 	aliases_.clear();
 	hashranges_.clear();
@@ -135,10 +145,13 @@ bool TextureReplacer::LoadIni() {
 	// First, check for textures.zip, which is used to reduce IO.
 	VFSBackend *dir = ZipFileReader::Create(zipPath, "", false);
 	if (!dir) {
-		INFO_LOG(Log::G3D, "%s wasn't a zip file - opening the directory %s instead.", zipPath.c_str(), basePath_.c_str());
+		INFO_LOG(Log::TexReplacement, "%s wasn't a zip file - opening the directory %s instead.", zipPath.c_str(), basePath_.c_str());
 		vfsIsZip_ = false;
 		dir = new DirectoryReader(basePath_);
 	} else {
+		if (!replaceEnabled_ && saveEnabled_) {
+			WARN_LOG(Log::TexReplacement, "Found zip file even though only saving is enabled! This is weird.");
+		}
 		vfsIsZip_ = true;
 	}
 
@@ -146,7 +159,7 @@ bool TextureReplacer::LoadIni() {
 	bool iniLoaded = ini.LoadFromVFS(*dir, INI_FILENAME);
 
 	if (iniLoaded) {
-		if (!LoadIniValues(ini, dir)) {
+		if (!LoadIniValues(ini, dir, false, error)) {
 			delete dir;
 			return false;
 		}
@@ -158,15 +171,17 @@ bool TextureReplacer::LoadIni() {
 				IniFile overrideIni;
 				iniLoaded = overrideIni.LoadFromVFS(*dir, overrideFilename);
 				if (!iniLoaded) {
-					ERROR_LOG(Log::G3D, "Failed to load extra texture ini: %s", overrideFilename.c_str());
+					*error = "Loading override ini failed: " + overrideFilename;
+					ERROR_LOG(Log::TexReplacement, "Failed to load extra texture ini: %s", overrideFilename.c_str());
 					// Since this error is most likely to occure for texture pack creators, let's just bail here
 					// so that the creator is more likely to look in the logs for what happened.
 					delete dir;
 					return false;
 				}
 
-				INFO_LOG(Log::G3D, "Loading extra texture ini: %s", overrideFilename.c_str());
-				if (!LoadIniValues(overrideIni, nullptr, true)) {
+				INFO_LOG(Log::TexReplacement, "Loading extra texture ini: %s", overrideFilename.c_str());
+				if (!LoadIniValues(overrideIni, nullptr, true, error)) {
+					*error = "Override: " + *error;
 					delete dir;
 					return false;
 				}
@@ -175,17 +190,20 @@ bool TextureReplacer::LoadIni() {
 	} else {
 		if (vfsIsZip_) {
 			// We don't accept zip files without inis.
-			ERROR_LOG(Log::G3D, "Texture pack lacking ini file: %s", basePath_.c_str());
+			ERROR_LOG(Log::TexReplacement, "Texture pack lacking ini file: %s", basePath_.c_str());
+			*error = "Zip files without ini files will not load";
 			delete dir;
 			return false;
 		} else {
-			WARN_LOG(Log::G3D, "Texture pack lacking ini file: %s", basePath_.c_str());
+			if (replaceEnabled_) {
+				WARN_LOG(Log::TexReplacement, "Texture pack lacking ini file: %s  Proceeding with only hash-named textures in the root.", basePath_.c_str());
+			}
 			// Do what we can do anyway: Scan for textures and build the map.
 			std::map<ReplacementCacheKey, std::map<int, std::string>> filenameMap;
 			ScanForHashNamedFiles(dir, filenameMap);
 
 			if (filenameMap.empty()) {
-				WARN_LOG(Log::G3D, "No replacement textures found.");
+				WARN_LOG(Log::TexReplacement, "No replacement textures found.");
 				return false;
 			}
 
@@ -194,7 +212,9 @@ bool TextureReplacer::LoadIni() {
 	}
 
 	auto gr = GetI18NCategory(I18NCat::GRAPHICS);
-	g_OSD.Show(OSDType::MESSAGE_SUCCESS, gr->T("Texture replacement pack activated"), 2.0f);
+	if (replaceEnabled_) {
+		g_OSD.Show(OSDType::MESSAGE_SUCCESS, gr->T("Texture replacement pack activated"), 3.0f);
+	}
 
 	vfs_ = dir;
 
@@ -204,10 +224,12 @@ bool TextureReplacer::LoadIni() {
 		repl.second->vfs_ = vfs_;
 	}
 
-	if (vfsIsZip_) {
-		INFO_LOG(Log::G3D, "Texture pack activated from '%s'", (basePath_ / ZIP_FILENAME).c_str());
-	} else {
-		INFO_LOG(Log::G3D, "Texture pack activated from '%s'", basePath_.c_str());
+	if (replaceEnabled_) {
+		if (vfsIsZip_) {
+			INFO_LOG(Log::TexReplacement, "Texture pack activated from '%s'", (basePath_ / ZIP_FILENAME).c_str());
+		} else {
+			INFO_LOG(Log::TexReplacement, "Texture pack activated from '%s'", basePath_.c_str());
+		}
 	}
 
 	// The ini doesn't have to exist for the texture directory or zip to be valid.
@@ -216,6 +238,7 @@ bool TextureReplacer::LoadIni() {
 
 void TextureReplacer::ScanForHashNamedFiles(VFSBackend *dir, std::map<ReplacementCacheKey, std::map<int, std::string>> &filenameMap) {
 	// Scan the root of the texture folder/zip and preinitialize the hash map.
+	// TODO: Could put VFSFileReference into the map...
 	std::vector<File::FileInfo> filesInRoot;
 	dir->GetFileListing("", &filesInRoot, nullptr);
 	for (auto file : filesInRoot) {
@@ -235,7 +258,7 @@ void TextureReplacer::ScanForHashNamedFiles(VFSBackend *dir, std::map<Replacemen
 			ReplacementCacheKey key(0, 0);
 			int level = 0;  // sscanf might fail to pluck the level, but that's ok, we default to 0. sscanf doesn't write to non-matched outputs.
 			if (sscanf(hash.c_str(), "%16llx%8x_%d", &key.cachekey, &key.hash, &level) >= 1) {
-				// INFO_LOG(Log::G3D, "hash-like file in root, adding: %s", file.name.c_str());
+				// INFO_LOG(Log::TexReplacement, "hash-like file in root, adding: %s", file.name.c_str());
 				filenameMap[key][level] = file.name;
 			}
 		}
@@ -251,12 +274,12 @@ void TextureReplacer::ComputeAliasMap(const std::map<ReplacementCacheKey, std::m
 				alias += level.second + "|";
 				mipIndex++;
 			} else {
-				WARN_LOG(Log::G3D, "Non-sequential mip index %d, breaking. filenames=%s", level.first, level.second.c_str());
+				WARN_LOG(Log::TexReplacement, "Non-sequential mip index %d, breaking. filenames=%s", level.first, level.second.c_str());
 				break;
 			}
 		}
 		if (alias == "|") {
-			alias = "";  // marker for no replacement
+			alias.clear();  // marker for no replacement
 		}
 		// Replace any '\' with '/', to be safe and consistent. Since these are from the ini file, we do this on all platforms.
 		for (auto &c : alias) {
@@ -268,10 +291,15 @@ void TextureReplacer::ComputeAliasMap(const std::map<ReplacementCacheKey, std::m
 	}
 }
 
-bool TextureReplacer::LoadIniValues(IniFile &ini, VFSBackend *dir, bool isOverride) {
+bool TextureReplacer::LoadIniValues(IniFile &ini, VFSBackend *dir, bool isOverride, std::string *error) {
+	INFO_LOG(Log::G3D, "Loading ini values...");
+
 	auto options = ini.GetOrCreateSection("options");
 	std::string hash;
-	options->Get("hash", &hash, "");
+	if (!options->Get("hash", &hash, "")) {
+		*error = "textures.ini: Hash type not specified";
+		return false;
+	}
 	if (strcasecmp(hash.c_str(), "quick") == 0) {
 		hash_ = ReplacedTextureHash::QUICK;
 	} else if (strcasecmp(hash.c_str(), "xxh32") == 0) {
@@ -279,7 +307,7 @@ bool TextureReplacer::LoadIniValues(IniFile &ini, VFSBackend *dir, bool isOverri
 	} else if (strcasecmp(hash.c_str(), "xxh64") == 0) {
 		hash_ = ReplacedTextureHash::XXH64;
 	} else if (!isOverride || !hash.empty()) {
-		ERROR_LOG(Log::G3D, "Unsupported hash type: %s", hash.c_str());
+		*error = "textures.ini: Unsupported hash type: " + hash;
 		return false;
 	}
 
@@ -288,19 +316,20 @@ bool TextureReplacer::LoadIniValues(IniFile &ini, VFSBackend *dir, bool isOverri
 	// Multiplies sizeInRAM/bytesPerLine in XXHASH by 0.5.
 	options->Get("reduceHash", &reduceHash_, reduceHash_);
 	options->Get("ignoreMipmap", &ignoreMipmap_, ignoreMipmap_);
+	options->Get("skipLastDXT1Blocks128x64", &skipLastDXT1Blocks128x64_, skipLastDXT1Blocks128x64_);
 	if (reduceHash_ && hash_ == ReplacedTextureHash::QUICK) {
 		reduceHash_ = false;
-		ERROR_LOG(Log::G3D, "Texture Replacement: reduceHash option requires safer hash, use xxh32 or xxh64 instead.");
+		ERROR_LOG(Log::TexReplacement, "Texture Replacement: reduceHash option requires safer hash, use xxh32 or xxh64 instead.");
 	}
 
 	if (ignoreAddress_ && hash_ == ReplacedTextureHash::QUICK) {
 		ignoreAddress_ = false;
-		ERROR_LOG(Log::G3D, "Texture Replacement: ignoreAddress option requires safer hash, use xxh32 or xxh64 instead.");
+		ERROR_LOG(Log::TexReplacement, "Texture Replacement: ignoreAddress option requires safer hash, use xxh32 or xxh64 instead.");
 	}
 
 	int version = 0;
 	if (options->Get("version", &version, 0) && version > VERSION) {
-		ERROR_LOG(Log::G3D, "Unsupported texture replacement version %d, trying anyway", version);
+		ERROR_LOG(Log::TexReplacement, "Unsupported texture replacement version %d, trying anyway", version);
 	}
 
 	int badFileNameCount = 0;
@@ -314,16 +343,21 @@ bool TextureReplacer::LoadIniValues(IniFile &ini, VFSBackend *dir, bool isOverri
 	std::string badFilenames;
 
 	if (ini.HasSection("hashes")) {
-		auto hashes = ini.GetOrCreateSection("hashes")->ToMap();
+		const Section *hashesSection = ini.GetOrCreateSection("hashes");
 		// Format: hashname = filename.png
 		bool checkFilenames = saveEnabled_ && !g_Config.bIgnoreTextureFilenames && !vfsIsZip_;
 
-		for (const auto &[k, v] : hashes) {
+		for (const auto &line : hashesSection->Lines()) {
+			if (line.Key().empty())
+				continue;
 			ReplacementCacheKey key(0, 0);
 			// sscanf might fail to pluck the level if omitted from the line, but that's ok, we default level to 0.
 			// sscanf doesn't write to non-matched outputs.
 			int level = 0;
-			if (sscanf(k.c_str(), "%16llx%8x_%d", &key.cachekey, &key.hash, &level) >= 1) {
+			char k[128];
+			truncate_cpy(k, line.Key());
+			std::string_view v = line.Value();
+			if (sscanf(k, "%16llx%8x_%d", &key.cachekey, &key.hash, &level) >= 1) {
 				// We allow empty filenames, to mark textures that we don't want to keep saving.
 				filenameMap[key][level] = v;
 				if (checkFilenames) {
@@ -345,10 +379,8 @@ bool TextureReplacer::LoadIniValues(IniFile &ini, VFSBackend *dir, bool isOverri
 						}
 					}
 				}
-			} else if (k.empty()) {
-				INFO_LOG(Log::G3D, "Ignoring [hashes] line with empty key: '= %s'", v.c_str());
 			} else {
-				ERROR_LOG(Log::G3D, "Unsupported syntax under [hashes], ignoring: %s = ", k.c_str());
+				ERROR_LOG(Log::TexReplacement, "Unsupported syntax under [hashes], ignoring: %s = ", k);
 			}
 		}
 	}
@@ -359,7 +391,7 @@ bool TextureReplacer::LoadIniValues(IniFile &ini, VFSBackend *dir, bool isOverri
 	if (badFileNameCount > 0) {
 		auto err = GetI18NCategory(I18NCat::ERRORS);
 		g_OSD.Show(OSDType::MESSAGE_WARNING, err->T("textures.ini filenames may not be cross - platform(banned characters)"), badFilenames, 6.0f);
-		WARN_LOG(Log::G3D, "Potentially bad filenames: %s", badFilenames.c_str());
+		WARN_LOG(Log::TexReplacement, "Potentially bad filenames: %s", badFilenames.c_str());
 	}
 
 	if (ini.HasSection("hashranges")) {
@@ -396,7 +428,7 @@ void TextureReplacer::ParseHashRange(const std::string &key, const std::string &
 	SplitString(value, ',', valueParts);
 
 	if (keyParts.size() != 3 || valueParts.size() != 2) {
-		ERROR_LOG(Log::G3D, "Ignoring invalid hashrange %s = %s, expecting addr,w,h = w,h", key.c_str(), value.c_str());
+		ERROR_LOG(Log::TexReplacement, "Ignoring invalid hashrange %s = %s, expecting addr,w,h = w,h", key.c_str(), value.c_str());
 		return;
 	}
 
@@ -409,19 +441,19 @@ void TextureReplacer::ParseHashRange(const std::string &key, const std::string &
 	u32 fromW;
 	u32 fromH;
 	if (!TryParse(keyParts[0], &addr) || !TryParse(keyParts[1], &fromW) || !TryParse(keyParts[2], &fromH)) {
-		ERROR_LOG(Log::G3D, "Ignoring invalid hashrange %s = %s, key format is 0x12345678,512,512", key.c_str(), value.c_str());
+		ERROR_LOG(Log::TexReplacement, "Ignoring invalid hashrange %s = %s, key format is 0x12345678,512,512", key.c_str(), value.c_str());
 		return;
 	}
 
 	u32 toW;
 	u32 toH;
 	if (!TryParse(valueParts[0], &toW) || !TryParse(valueParts[1], &toH)) {
-		ERROR_LOG(Log::G3D, "Ignoring invalid hashrange %s = %s, value format is 512,512", key.c_str(), value.c_str());
+		ERROR_LOG(Log::TexReplacement, "Ignoring invalid hashrange %s = %s, value format is 512,512", key.c_str(), value.c_str());
 		return;
 	}
 
 	if (toW > fromW || toH > fromH) {
-		ERROR_LOG(Log::G3D, "Ignoring invalid hashrange %s = %s, range bigger than source", key.c_str(), value.c_str());
+		ERROR_LOG(Log::TexReplacement, "Ignoring invalid hashrange %s = %s, range bigger than source", key.c_str(), value.c_str());
 		return;
 	}
 
@@ -439,10 +471,10 @@ void TextureReplacer::ParseFiltering(const std::string &key, const std::string &
 		} else if (!strcasecmp(value.c_str(), "auto")) {
 			filtering_[itemKey] = TEX_FILTER_AUTO;
 		} else {
-			ERROR_LOG(Log::G3D, "Unsupported syntax under [filtering]: %s", value.c_str());
+			ERROR_LOG(Log::TexReplacement, "Unsupported syntax under [filtering]: %s", value.c_str());
 		}
 	} else {
-		ERROR_LOG(Log::G3D, "Unsupported syntax under [filtering]: %s", key.c_str());
+		ERROR_LOG(Log::TexReplacement, "Unsupported syntax under [filtering]: %s", key.c_str());
 	}
 }
 
@@ -453,25 +485,25 @@ void TextureReplacer::ParseReduceHashRange(const std::string& key, const std::st
 	SplitString(value, ',', valueParts);
 
 	if (keyParts.size() != 2 || valueParts.size() != 1) {
-		ERROR_LOG(Log::G3D, "Ignoring invalid reducehashrange %s = %s, expecting w,h = reducehashvalue", key.c_str(), value.c_str());
+		ERROR_LOG(Log::TexReplacement, "Ignoring invalid reducehashrange %s = %s, expecting w,h = reducehashvalue", key.c_str(), value.c_str());
 		return;
 	}
 
 	u32 forW;
 	u32 forH;
 	if (!TryParse(keyParts[0], &forW) || !TryParse(keyParts[1], &forH)) {
-		ERROR_LOG(Log::G3D, "Ignoring invalid reducehashrange %s = %s, key format is 512,512", key.c_str(), value.c_str());
+		ERROR_LOG(Log::TexReplacement, "Ignoring invalid reducehashrange %s = %s, key format is 512,512", key.c_str(), value.c_str());
 		return;
 	}
 
 	float rhashvalue;
 	if (!TryParse(valueParts[0], &rhashvalue)) {
-		ERROR_LOG(Log::G3D, "Ignoring invalid reducehashrange %s = %s, value format is 0.5", key.c_str(), value.c_str());
+		ERROR_LOG(Log::TexReplacement, "Ignoring invalid reducehashrange %s = %s, value format is 0.5", key.c_str(), value.c_str());
 		return;
 	}
 
 	if (rhashvalue == 0) {
-		ERROR_LOG(Log::G3D, "Ignoring invalid hashrange %s = %s, reducehashvalue can't be 0", key.c_str(), value.c_str());
+		ERROR_LOG(Log::TexReplacement, "Ignoring invalid hashrange %s = %s, reducehashvalue can't be 0", key.c_str(), value.c_str());
 		return;
 	}
 
@@ -493,14 +525,29 @@ u32 TextureReplacer::ComputeHash(u32 addr, int bufw, int w, int h, bool swizzled
 	}
 
 	const u8 *checkp = Memory::GetPointerUnchecked(addr);
+
+	float reduceHashSize = 1.0f;
 	if (reduceHash_) {
 		reduceHashSize = LookupReduceHashRange(w, h);
 		// default to reduceHashGlobalValue which default is 0.5
 	}
+
 	if (bufw <= w) {
 		// We can assume the data is contiguous.  These are the total used pixels.
 		const u32 totalPixels = bufw * h + (w - bufw);
-		const u32 sizeInRAM = (textureBitsPerPixel[fmt] * totalPixels) / 8 * reduceHashSize;
+		u32 sizeInRAM = (textureBitsPerPixel[fmt] * totalPixels) / 8 * reduceHashSize;
+
+		// Sanity check: Ignore textures that are at the end of RAM.
+		if (Memory::MaxSizeAtAddress(addr) < sizeInRAM) {
+			ERROR_LOG(Log::G3D, "Can't hash a %d bytes textures at %08x - end point is outside memory", sizeInRAM, addr);
+			return 0;
+		}
+
+		// Hack for Yu Gi Oh texture hashing problem. See issue #19714
+		if (skipLastDXT1Blocks128x64_ && fmt == GE_TFMT_DXT1 && w == 128 && h == 64) {
+			// Skip the last few blocks as specified.
+			sizeInRAM -= 8 * skipLastDXT1Blocks128x64_;
+		}
 
 		switch (hash_) {
 		case ReplacedTextureHash::QUICK:
@@ -582,7 +629,7 @@ ReplacedTexture *TextureReplacer::FindReplacement(u64 cachekey, u32 hash, int w,
 
 	// Early-out for ignored textures, let's not bother even starting a thread task.
 	if (ignored) {
-		// WARN_LOG(Log::G3D, "Not found/ignored: %s (%d, %d)", hashfiles.c_str(), (int)foundReplacement, (int)ignored);
+		// WARN_LOG(Log::TexReplacement, "Not found/ignored: %s (%d, %d)", hashfiles.c_str(), (int)foundReplacement, (int)ignored);
 		// Insert an entry into the cache for faster lookup next time.
 		ReplacedTextureRef ref{};
 		cache_.emplace(std::make_pair(replacementKey, ref));
@@ -645,7 +692,7 @@ ReplacedTexture *TextureReplacer::FindReplacement(u64 cachekey, u32 hash, int w,
 static bool WriteTextureToPNG(png_imagep image, const Path &filename, int convert_to_8bit, const void *buffer, png_int_32 row_stride, const void *colormap) {
 	FILE *fp = File::OpenCFile(filename, "wb");
 	if (!fp) {
-		ERROR_LOG(Log::IO, "Unable to open texture file '%s' for writing.", filename.c_str());
+		ERROR_LOG(Log::TexReplacement, "Save texture: Unable to open texture file '%s' for writing.", filename.c_str());
 		return false;
 	}
 
@@ -653,7 +700,8 @@ static bool WriteTextureToPNG(png_imagep image, const Path &filename, int conver
 		fclose(fp);
 		return true;
 	} else {
-		ERROR_LOG(Log::System, "Texture PNG encode failed.");
+		// This shouldn't really happen.
+		ERROR_LOG(Log::TexReplacement, "Texture PNG encode failed.");
 		fclose(fp);
 		remove(filename.c_str());
 		return false;
@@ -717,11 +765,11 @@ public:
 		bool success = WriteTextureToPNG(&png, saveFilename, 0, rgbaData, w * 4, nullptr);
 		png_image_free(&png);
 		if (png.warning_or_error >= 2) {
-			ERROR_LOG(Log::G3D, "Saving texture to PNG produced errors.");
+			ERROR_LOG(Log::TexReplacement, "Saving texture to PNG produced errors.");
 		} else if (success) {
-			NOTICE_LOG(Log::G3D, "Saving texture for replacement: %08x / %dx%d in '%s'", replacedInfoHash, w, h, saveFilename.ToVisualString().c_str());
+			NOTICE_LOG(Log::TexReplacement, "Saving texture for replacement: %08x / %dx%d in '%s'", replacedInfoHash, w, h, saveFilename.ToVisualString().c_str());
 		} else {
-			ERROR_LOG(Log::G3D, "Failed to write '%s'", saveFilename.c_str());
+			ERROR_LOG(Log::TexReplacement, "Failed to write '%s'", saveFilename.c_str());
 		}
 	}
 };
@@ -742,6 +790,7 @@ void TextureReplacer::NotifyTextureDecoded(ReplacedTexture *texture, const Repla
 	_assert_msg_(saveEnabled_, "Texture saving not enabled");
 	_assert_(srcPitch >= 0);
 	_assert_(data);
+	_assert_(level >= 0);
 
 	if (!WillSave(replacedInfo)) {
 		// Ignore.
@@ -807,7 +856,7 @@ void TextureReplacer::NotifyTextureDecoded(ReplacedTexture *texture, const Repla
 	size_t saveBufSize = w * h * 4;
 	u8 *saveBuf = (u8 *)malloc(saveBufSize);
 	if (!saveBuf) {
-		ERROR_LOG(Log::G3D, "Failed to allocated %d bytes of memory for saving a texture", (int)saveBufSize);
+		ERROR_LOG(Log::TexReplacement, "Failed to allocated %d bytes of memory for saving a texture", (int)saveBufSize);
 		return;
 	}
 
@@ -864,7 +913,7 @@ void TextureReplacer::Decimate(ReplacerDecimateMode mode) {
 
 	double totalSizeGB = totalSize / (1024.0 * 1024.0 * 1024.0);
 	if (totalSizeGB >= 1.0) {
-		WARN_LOG(Log::G3D, "Decimated replacements older than %fs, currently using %f GB of RAM", age, totalSizeGB);
+		WARN_LOG(Log::TexReplacement, "Decimated replacements older than %fs, currently using %f GB of RAM", age, totalSizeGB);
 	}
 	lastTextureCacheSizeGB_ = totalSizeGB;
 }
