@@ -1,6 +1,7 @@
 use std::io;
 
 mod section;
+use section::{line_value, Section};
 
 mod inifile;
 use inifile::IniFile;
@@ -13,6 +14,8 @@ struct Args {
     cmd: Command,
     #[arg(short, long)]
     dry_run: bool,
+    #[arg(short, long)]
+    verbose: bool,
 }
 
 #[derive(Parser, Debug)]
@@ -42,6 +45,11 @@ enum Command {
         new: String,
         key: String,
     },
+    DupeKey {
+        section: String,
+        old: String,
+        new: String,
+    },
     RenameKey {
         section: String,
         old: String,
@@ -54,6 +62,12 @@ enum Command {
         section: String,
         key: String,
     },
+    GetNewKeys,
+    ImportSingle {
+        filename: String,
+        section: String,
+        key: String,
+    },
 }
 
 fn copy_missing_lines(
@@ -100,6 +114,25 @@ fn deal_with_unknown_lines(
     Ok(())
 }
 
+fn print_keys_if_not_in(
+    reference_ini: &IniFile,
+    target_ini: &mut IniFile,
+    header: &str,
+) -> io::Result<()> {
+    for reference_section in &reference_ini.sections {
+        if let Some(target_section) = target_ini.get_section_mut(&reference_section.name) {
+            let keys = target_section.get_keys_if_not_in(reference_section);
+            if !keys.is_empty() {
+                println!("{} ({})", reference_section.name, header);
+                for key in &keys {
+                    println!("- {}", key);
+                }
+            }
+        }
+    }
+    Ok(())
+}
+
 fn move_key(target_ini: &mut IniFile, old: &str, new: &str, key: &str) -> io::Result<()> {
     if let Some(old_section) = target_ini.get_section_mut(old) {
         if let Some(line) = old_section.remove_line(key) {
@@ -161,6 +194,15 @@ fn rename_key(target_ini: &mut IniFile, section: &str, old: &str, new: &str) ->
     Ok(())
 }
 
+fn dupe_key(target_ini: &mut IniFile, section: &str, old: &str, new: &str) -> io::Result<()> {
+    if let Some(section) = target_ini.get_section_mut(section) {
+        section.dupe_key(old, new);
+    } else {
+        println!("No section {}", section);
+    }
+    Ok(())
+}
+
 fn sort_section(target_ini: &mut IniFile, section: &str) -> io::Result<()> {
     if let Some(section) = target_ini.get_section_mut(section) {
         section.sort();
@@ -201,6 +243,25 @@ fn main() {
         }
     }
 
+    let mut single_ini_section: Option<Section> = None;
+    if let Command::ImportSingle {
+        filename,
+        section,
+        key,
+    } = &opt.cmd
+    {
+        if let Ok(single_ini) = IniFile::parse(filename) {
+            if let Some(single_section) = single_ini.get_section("Single") {
+                single_ini_section = Some(single_section.clone());
+            } else {
+                println!("No section {} in {}", section, filename);
+            }
+        } else {
+            println!("Failed to parse {}", filename);
+            return;
+        }
+    }
+
     for filename in filenames {
         let reference_ini = &reference_ini;
         if filename == "langtool" {
@@ -208,7 +269,9 @@ fn main() {
             continue;
         }
         let target_ini_filename = format!("{}/{}", root, filename);
-        println!("Langtool processing {}", target_ini_filename);
+        if opt.verbose {
+            println!("Langtool processing {}", target_ini_filename);
+        }
 
         let mut target_ini = IniFile::parse(&target_ini_filename).unwrap();
 
@@ -224,6 +287,9 @@ fn main() {
             Command::RemoveUnknownLines {} => {
                 deal_with_unknown_lines(reference_ini, &mut target_ini, true).unwrap();
             }
+            Command::GetNewKeys {} => {
+                print_keys_if_not_in(reference_ini, &mut target_ini, &target_ini_filename).unwrap();
+            }
             Command::SortSection { ref section } => sort_section(&mut target_ini, section).unwrap(),
             Command::RenameKey {
                 ref section,
@@ -247,18 +313,57 @@ fn main() {
                 move_key(&mut target_ini, old, new, key).unwrap();
             }
             Command::CopyKey {
+                // Copies between sections
                 ref old,
                 ref new,
                 ref key,
             } => {
                 copy_key(&mut target_ini, old, new, key).unwrap();
             }
+            Command::DupeKey {
+                ref section,
+                ref old,
+                ref new,
+            } => {
+                dupe_key(&mut target_ini, section, old, new).unwrap();
+            }
             Command::RemoveKey {
                 ref section,
                 ref key,
             } => {
                 remove_key(&mut target_ini, section, key).unwrap();
             }
+            Command::ImportSingle {
+                filename: _,
+                ref section,
+                ref key,
+            } => {
+                let lang_id = filename.strip_suffix(".ini").unwrap();
+                if let Some(single_section) = &single_ini_section {
+                    if let Some(target_section) = target_ini.get_section_mut(&section) {
+                        if let Some(single_line) = single_section.get_line(&lang_id) {
+                            if let Some(value) = line_value(&single_line) {
+                                println!(
+                                    "Inserting value {} for key {} in section {} in {}",
+                                    value, key, section, target_ini_filename
+                                );
+                                if !target_section
+                                    .insert_line_if_missing(&format!("{} = {}", key, value))
+                                {
+                                    // Didn't insert it, so it exists. We need to replace it.
+                                    target_section.set_value(key, value);
+                                }
+                            }
+                        } else {
+                            println!("No lang_id {} in single section", lang_id);
+                        }
+                    } else {
+                        println!("No section {} in {}", section, target_ini_filename);
+                    }
+                } else {
+                    println!("No section {} in {}", section, filename);
+                }
+            }
         }
 
         if !opt.dry_run {
@@ -302,12 +407,21 @@ fn main() {
             move_key(&mut reference_ini, old, new, key).unwrap();
         }
         Command::CopyKey {
+            // between sections
             ref old,
             ref new,
             ref key,
         } => {
             copy_key(&mut reference_ini, old, new, key).unwrap();
         }
+        Command::DupeKey {
+            // Inside a section, preserving a value
+            ref section,
+            ref old,
+            ref new,
+        } => {
+            dupe_key(&mut reference_ini, section, old, new).unwrap();
+        }
         Command::RemoveKey {
             ref section,
             ref key,
