@@ -6,26 +6,18 @@
 #include "Common/System/Display.h"
 #include "UI/TabbedDialogScreen.h"
 
-UI::LinearLayout *TabbedUIDialogScreenWithGameBackground::AddTab(const char *tag, std::string_view title, bool isSearch) {
+void TabbedUIDialogScreenWithGameBackground::AddTab(const char *tag, std::string_view title, std::function<void(UI::LinearLayout *)> createCallback, bool isSearch) {
 	using namespace UI;
-	ViewGroup *scroll = new ScrollView(ORIENT_VERTICAL, new LinearLayoutParams(FILL_PARENT, FILL_PARENT));
-	scroll->SetTag(tag);
 
-	LinearLayout *contents = new LinearLayoutList(ORIENT_VERTICAL);
-	contents->SetSpacing(0);
-	scroll->Add(contents);
-	tabHolder_->AddTab(title, scroll);
-
-	if (!isSearch) {
-		settingTabContents_.push_back(contents);
-
-		auto se = GetI18NCategory(I18NCat::SEARCH);
-		auto notice = contents->Add(new TextView(se->T("Filtering settings by '%1'"), new LinearLayoutParams(Margins(20, 5))));
-		notice->SetVisibility(V_GONE);
-		settingTabFilterNotices_.push_back(notice);
-	}
-
-	return contents;
+	tabHolder_->AddTabDeferred(title, [createCallback = std::move(createCallback), tag]() -> UI::ViewGroup * {
+		ViewGroup *scroll = new ScrollView(ORIENT_VERTICAL, new LinearLayoutParams(FILL_PARENT, FILL_PARENT));
+		scroll->SetTag(tag);
+		LinearLayout *contents = new LinearLayoutList(ORIENT_VERTICAL);
+		contents->SetSpacing(0);
+		scroll->Add(contents);
+		createCallback(contents);
+		return scroll;
+	});
 }
 
 void TabbedUIDialogScreenWithGameBackground::CreateViews() {
@@ -40,22 +32,27 @@ void TabbedUIDialogScreenWithGameBackground::CreateViews() {
 
 	root_ = new AnchorLayout(new LayoutParams(FILL_PARENT, FILL_PARENT));
 
+	auto se = GetI18NCategory(I18NCat::SEARCH);
+	filterNotice_ = new TextView("(filter notice, you shouldn't see this text", new LinearLayoutParams(Margins(20, 5)));
+	filterNotice_->SetVisibility(V_GONE);
+
 	if (vertical) {
 		auto di = GetI18NCategory(I18NCat::DIALOG);
 		LinearLayout *verticalLayout = new LinearLayout(ORIENT_VERTICAL, new LayoutParams(FILL_PARENT, FILL_PARENT));
-		tabHolder_ = new TabHolder(ORIENT_HORIZONTAL, 200, new LinearLayoutParams(1.0f));
+		tabHolder_ = new TabHolder(ORIENT_HORIZONTAL, 200, filterNotice_, new LinearLayoutParams(1.0f));
 		verticalLayout->Add(tabHolder_);
+		CreateExtraButtons(verticalLayout, 0);
 		verticalLayout->Add(new Choice(di->T("Back"), "", false, new LinearLayoutParams(FILL_PARENT, WRAP_CONTENT, 0.0f, Margins(0))))->OnClick.Handle<UIScreen>(this, &UIScreen::OnBack);
 		root_->Add(verticalLayout);
 	} else {
-		tabHolder_ = new TabHolder(ORIENT_VERTICAL, 200, new AnchorLayoutParams(10, 0, 10, 0, false));
+		tabHolder_ = new TabHolder(ORIENT_VERTICAL, 200, filterNotice_, new AnchorLayoutParams(10, 0, 10, 0, false));
+		CreateExtraButtons(tabHolder_->Container(), 10);
 		tabHolder_->AddBack(this);
 		root_->Add(tabHolder_);
 	}
+
 	tabHolder_->SetTag(tag());  // take the tag from the screen.
 	root_->SetDefaultFocusView(tabHolder_);
-	settingTabContents_.clear();
-	settingTabFilterNotices_.clear();
 
 	float leftSide = 40.0f;
 	if (!vertical) {
@@ -78,26 +75,26 @@ void TabbedUIDialogScreenWithGameBackground::CreateViews() {
 		// Hide search if screen is too small.
 		int deviceType = System_GetPropertyInt(SYSPROP_DEVICE_TYPE);
 		if ((g_display.dp_xres < g_display.dp_yres || g_display.dp_yres >= 500) && (deviceType != DEVICE_TYPE_VR) && ShowSearchControls()) {
-			auto se = GetI18NCategory(I18NCat::SEARCH);
-			auto ms = GetI18NCategory(I18NCat::MAINSETTINGS);
 			// Search
-			LinearLayout *searchSettings = AddTab("GameSettingsSearch", ms->T("Search"), true);
+			auto ms = GetI18NCategory(I18NCat::MAINSETTINGS);
+			AddTab("GameSettingsSearch", ms->T("Search"), [this](UI::LinearLayout *searchSettings) {
+				auto se = GetI18NCategory(I18NCat::SEARCH);
 
-			searchSettings->Add(new ItemHeader(se->T("Find settings")));
-			searchSettings->Add(new PopupTextInputChoice(GetRequesterToken(), &searchFilter_, se->T("Filter"), "", 64, screenManager()))->OnChange.Add([=](UI::EventParams &e) {
-				System_PostUIMessage(UIMessage::GAMESETTINGS_SEARCH, StripSpaces(searchFilter_));
-				return UI::EVENT_DONE;
-			});
+				searchSettings->Add(new ItemHeader(se->T("Find settings")));
+				searchSettings->Add(new PopupTextInputChoice(GetRequesterToken(), &searchFilter_, se->T("Filter"), "", 64, screenManager()))->OnChange.Add([=](UI::EventParams &e) {
+					System_PostUIMessage(UIMessage::GAMESETTINGS_SEARCH, StripSpaces(searchFilter_));
+					return UI::EVENT_DONE;
+				});
 
-			clearSearchChoice_ = searchSettings->Add(new Choice(se->T("Clear filter")));
-			clearSearchChoice_->OnClick.Add([=](UI::EventParams &e) {
-				System_PostUIMessage(UIMessage::GAMESETTINGS_SEARCH, "");
-				return UI::EVENT_DONE;
-			});
+				clearSearchChoice_ = searchSettings->Add(new Choice(se->T("Clear filter")));
+				clearSearchChoice_->OnClick.Add([=](UI::EventParams &e) {
+					System_PostUIMessage(UIMessage::GAMESETTINGS_SEARCH, "");
+					return UI::EVENT_DONE;
+				});
+				clearSearchChoice_->SetVisibility(searchFilter_.empty() ? UI::V_GONE : UI::V_VISIBLE);
 
-			noSearchResults_ = searchSettings->Add(new TextView(se->T("No settings matched '%1'"), new LinearLayoutParams(Margins(20, 5))));
-
-			ApplySearchFilter();
+				noSearchResults_ = searchSettings->Add(new TextView("", new LinearLayoutParams(Margins(20, 5))));
+			}, true);
 		}
 	}
 }
@@ -118,17 +115,45 @@ void TabbedUIDialogScreenWithGameBackground::RecreateViews() {
 	UIScreen::RecreateViews();
 }
 
+void TabbedUIDialogScreenWithGameBackground::EnsureTabs() {
+	_assert_(tabHolder_);
+	tabHolder_->EnsureAllCreated();
+}
+
 void TabbedUIDialogScreenWithGameBackground::ApplySearchFilter() {
+	using namespace UI;
 	auto se = GetI18NCategory(I18NCat::SEARCH);
 
-	bool matches = searchFilter_.empty();
-	for (int t = 0; t < (int)settingTabContents_.size(); ++t) {
-		auto tabContents = settingTabContents_[t];
-		bool tabMatches = searchFilter_.empty();
+	EnsureTabs();
 
-		// Show an indicator that a filter is applied.
-		settingTabFilterNotices_[t]->SetVisibility(tabMatches ? UI::V_GONE : UI::V_VISIBLE);
-		settingTabFilterNotices_[t]->SetText(ApplySafeSubstitutions(se->T("Filtering settings by '%1'"), searchFilter_));
+	// Show an indicator that a filter is applied.
+	filterNotice_->SetVisibility(searchFilter_.empty() ? UI::V_GONE : UI::V_VISIBLE);
+	filterNotice_->SetText(ApplySafeSubstitutions(se->T("Filtering settings by '%1'"), searchFilter_));
+
+	bool matches = searchFilter_.empty();
+
+	const std::vector<ViewGroup *> settingTabs = tabHolder_->GetTabContentViews();
+
+	for (int t = 0; t < (int)settingTabs.size(); ++t) {
+		const ViewGroup *tabContents = settingTabs[t];
+		std::string_view tag = tabContents->Tag();
+
+		// Dive down to the actual list of settings.
+		// TODO: Do this recursively instead.
+		while (tabContents->GetNumSubviews() == 1) {
+			View *v = tabContents->GetViewByIndex(0);
+			if (v->IsViewGroup()) {
+				tabContents = (ViewGroup *)v;
+			} else {
+				break;
+			}
+		}
+
+		if (tag == "GameSettingsSearch") {
+			continue;
+		}
+
+		bool tabMatches = searchFilter_.empty();
 
 		UI::View *lastHeading = nullptr;
 		for (int i = 1; i < tabContents->GetNumSubviews(); ++i) {
