@@ -37,10 +37,12 @@
 #include "Common/TimeUtil.h"
 #include "Common/StringUtils.h"
 #include "Common/Data/Text/I18n.h"
+#include "Common/Data/Text/Parsers.h"
 #include "Common/Input/InputState.h"
 #include "Common/Input/KeyCodes.h"
 #include "Common/Thread/ThreadUtil.h"
 #include "Common/Data/Encoding/Utf8.h"
+#include "ext/imgui/imgui_impl_platform.h"
 
 #include "Core/Core.h"
 #include "Core/Config.h"
@@ -81,9 +83,11 @@
 #include "Windows/CaptureDevice.h"
 #include "Windows/TouchInputHandler.h"
 #include "Windows/MainWindowMenu.h"
-#include "GPU/GPUInterface.h"
+#include "GPU/GPUCommon.h"
 #include "UI/OnScreenDisplay.h"
 #include "UI/GameSettingsScreen.h"
+#include "UI/PauseScreen.h"
+#include "Core/SaveState.h"
 
 #define MOUSEEVENTF_FROMTOUCH_NOPEN 0xFF515780 //http://msdn.microsoft.com/en-us/library/windows/desktop/ms703320(v=vs.85).aspx
 #define MOUSEEVENTF_MASK_PLUS_PENTOUCH 0xFFFFFF80
@@ -109,7 +113,8 @@ struct VerySleepy_AddrInfo {
 	wchar_t name[256];
 };
 
-static std::wstring windowTitle;
+static std::mutex g_windowTitleLock;
+static std::wstring g_windowTitle;
 
 #define TIMER_CURSORUPDATE 1
 #define TIMER_CURSORMOVEUPDATE 2
@@ -273,7 +278,7 @@ namespace MainWindow
 
 	static void HandleSizeChange(int newSizingType) {
 		SavePosition();
-		Core_NotifyWindowHidden(false);
+		Native_NotifyWindowHidden(false);
 		if (!g_Config.bPauseWhenMinimized) {
 			System_PostUIMessage(UIMessage::WINDOW_MINIMIZED, "false");
 		}
@@ -294,7 +299,7 @@ namespace MainWindow
 
 		DEBUG_LOG(Log::System, "Pixel width/height: %dx%d", PSP_CoreParameter().pixelWidth, PSP_CoreParameter().pixelHeight);
 
-		if (UpdateScreenScale(width, height)) {
+		if (Native_UpdateScreenScale(width, height, UIScaleFactorToMultiplier(g_Config.iUIScaleFactor))) {
 			System_PostUIMessage(UIMessage::GPU_DISPLAY_RESIZED);
 			System_PostUIMessage(UIMessage::GPU_RENDER_RESIZED);
 		}
@@ -465,7 +470,11 @@ namespace MainWindow
 	}
 
 	void UpdateWindowTitle() {
-		std::wstring title = windowTitle;
+		std::wstring title;
+		{
+			std::lock_guard<std::mutex> lock(g_windowTitleLock);
+			title = g_windowTitle;
+		}
 		if (PPSSPP_ID >= 1 && GetInstancePeerCount() > 1) {
 			title.append(ConvertUTF8ToWString(StringFromFormat(" (instance: %d)", (int)PPSSPP_ID)));
 		}
@@ -473,7 +482,11 @@ namespace MainWindow
 	}
 
 	void SetWindowTitle(const wchar_t *title) {
-		windowTitle = title;
+		{
+			std::lock_guard<std::mutex> lock(g_windowTitleLock);
+			g_windowTitle = title;
+		}
+		PostMessage(MainWindow::GetHWND(), MainWindow::WM_USER_WINDOW_TITLE_CHANGED, 0, 0);
 	}
 
 	BOOL Show(HINSTANCE hInstance) {
@@ -611,6 +624,33 @@ namespace MainWindow
 		case WM_SETFOCUS:
 			break;
 
+		case WM_SETCURSOR:
+			if ((lParam & 0xFFFF) == HTCLIENT && g_Config.bShowImDebugger) {
+				LPTSTR win32_cursor = 0;
+				if (g_Config.bShowImDebugger) {
+					switch (ImGui_ImplPlatform_GetCursor()) {
+					case ImGuiMouseCursor_Arrow:        win32_cursor = IDC_ARROW; break;
+					case ImGuiMouseCursor_TextInput:    win32_cursor = IDC_IBEAM; break;
+					case ImGuiMouseCursor_ResizeAll:    win32_cursor = IDC_SIZEALL; break;
+					case ImGuiMouseCursor_ResizeEW:     win32_cursor = IDC_SIZEWE; break;
+					case ImGuiMouseCursor_ResizeNS:     win32_cursor = IDC_SIZENS; break;
+					case ImGuiMouseCursor_ResizeNESW:   win32_cursor = IDC_SIZENESW; break;
+					case ImGuiMouseCursor_ResizeNWSE:   win32_cursor = IDC_SIZENWSE; break;
+					case ImGuiMouseCursor_Hand:         win32_cursor = IDC_HAND; break;
+					case ImGuiMouseCursor_NotAllowed:   win32_cursor = IDC_NO; break;
+					}
+				}
+				if (win32_cursor) {
+					SetCursor(::LoadCursor(nullptr, win32_cursor));
+				} else {
+					SetCursor(nullptr);
+				}
+				return TRUE;
+			} else {
+				return DefWindowProc(hWnd, message, wParam, lParam);
+			}
+			break;
+
 		case WM_ERASEBKGND:
 			if (firstErase) {
 				firstErase = false;
@@ -632,29 +672,29 @@ namespace MainWindow
 				float y = GET_Y_LPARAM(lParam) * g_display.dpi_scale_y;
 				WindowsRawInput::SetMousePos(x, y);
 
-				TouchInput touch;
-				touch.id = 0;
-				touch.flags = TOUCH_DOWN;
+				TouchInput touch{};
+				touch.flags = TOUCH_DOWN | TOUCH_MOUSE;
+				touch.buttons = 1;
 				touch.x = x;
 				touch.y = y;
 				NativeTouch(touch);
 				SetCapture(hWnd);
 
 				// Simulate doubleclick, doesn't work with RawInput enabled
-				static double lastMouseDown;
+				static double lastMouseDownTime;
 				static float lastMouseDownX = -1.0f;
 				static float lastMouseDownY = -1.0f;
-				double now = time_now_d();
-				if ((now - lastMouseDown) < 0.001 * GetDoubleClickTime()) {
-					float dx = lastMouseDownX - x;
-					float dy = lastMouseDownY - y;
-					float distSq = dx * dx + dy * dy;
-					if (distSq < 3.0f*3.0f && !g_Config.bShowTouchControls && !g_Config.bMouseControl && GetUIState() == UISTATE_INGAME && g_Config.bFullscreenOnDoubleclick) {
+				const double now = time_now_d();
+				if ((now - lastMouseDownTime) < 0.001 * GetDoubleClickTime()) {
+					const float dx = lastMouseDownX - x;
+					const float dy = lastMouseDownY - y;
+					const float distSq = dx * dx + dy * dy;
+					if (distSq < 3.0f*3.0f && !g_Config.bShowTouchControls && !g_Config.bShowImDebugger && !g_Config.bMouseControl && GetUIState() == UISTATE_INGAME && g_Config.bFullscreenOnDoubleclick) {
 						SendToggleFullscreen(!g_Config.UseFullScreen());
 					}
-					lastMouseDown = 0.0;
+					lastMouseDownTime = 0.0;
 				} else {
-					lastMouseDown = now;
+					lastMouseDownTime = now;
 				}
 				lastMouseDownX = x;
 				lastMouseDownY = y;
@@ -680,14 +720,18 @@ namespace MainWindow
 				float y = (float)cursorY * g_display.dpi_scale_y;
 				WindowsRawInput::SetMousePos(x, y);
 
+				// Mouse moves now happen also when no button is pressed.
+				TouchInput touch{};
+				touch.flags = TOUCH_MOVE | TOUCH_MOUSE;
 				if (wParam & MK_LBUTTON) {
-					TouchInput touch;
-					touch.id = 0;
-					touch.flags = TOUCH_MOVE;
-					touch.x = x;
-					touch.y = y;
-					NativeTouch(touch);
+					touch.buttons |= 1;
 				}
+				if (wParam & MK_RBUTTON) {
+					touch.buttons |= 2;
+				}
+				touch.x = x;
+				touch.y = y;
+				NativeTouch(touch);
 			}
 			break;
 
@@ -702,9 +746,9 @@ namespace MainWindow
 				float y = (float)GET_Y_LPARAM(lParam) * g_display.dpi_scale_y;
 				WindowsRawInput::SetMousePos(x, y);
 
-				TouchInput touch;
-				touch.id = 0;
-				touch.flags = TOUCH_UP;
+				TouchInput touch{};
+				touch.buttons = 1;
+				touch.flags = TOUCH_UP | TOUCH_MOUSE;
 				touch.x = x;
 				touch.y = y;
 				NativeTouch(touch);
@@ -716,6 +760,34 @@ namespace MainWindow
 			touchHandler.handleTouchEvent(hWnd, message, wParam, lParam);
 			return 0;
 
+		case WM_RBUTTONDOWN:
+		{
+			float x = GET_X_LPARAM(lParam) * g_display.dpi_scale_x;
+			float y = GET_Y_LPARAM(lParam) * g_display.dpi_scale_y;
+
+			TouchInput touch{};
+			touch.buttons = 2;
+			touch.flags = TOUCH_DOWN | TOUCH_MOUSE;
+			touch.x = x;
+			touch.y = y;
+			NativeTouch(touch);
+			break;
+		}
+
+		case WM_RBUTTONUP:
+		{
+			float x = GET_X_LPARAM(lParam) * g_display.dpi_scale_x;
+			float y = GET_Y_LPARAM(lParam) * g_display.dpi_scale_y;
+
+			TouchInput touch{};
+			touch.buttons = 2;
+			touch.flags = TOUCH_UP | TOUCH_MOUSE;
+			touch.x = x;
+			touch.y = y;
+			NativeTouch(touch);
+			break;
+		}
+
 		default:
 			return DefWindowProc(hWnd, message, wParam, lParam);
 		}
@@ -753,6 +825,27 @@ namespace MainWindow
 		};
 	}
 
+	bool ConfirmAction(HWND hWnd, bool actionIsReset) {
+		const GlobalUIState state = GetUIState();
+		if (state == UISTATE_MENU || state == UISTATE_EXIT) {
+			return true;
+		}
+
+		std::string confirmExitMessage = GetConfirmExitMessage();
+		if (confirmExitMessage.empty()) {
+			return true;
+		}
+		auto di = GetI18NCategory(I18NCat::DIALOG);
+		auto mm = GetI18NCategory(I18NCat::MAINMENU);
+		if (!actionIsReset) {
+			confirmExitMessage += '\n';
+			confirmExitMessage += di->T("Are you sure you want to exit?");
+		} else {
+			// Reset is bit rarer, let's just omit the extra message for now.
+		}
+		return IDYES == MessageBox(hWnd, ConvertUTF8ToWString(confirmExitMessage).c_str(), ConvertUTF8ToWString(mm->T("Exit")).c_str(), MB_YESNO | MB_ICONQUESTION);
+	}
+
 	LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)	{
 		LRESULT darkResult = 0;
 		if (UAHDarkModeWndProc(hWnd, message, wParam, lParam, &darkResult)) {
@@ -768,6 +861,7 @@ namespace MainWindow
 			if (g_darkModeSupported) {
 				SendMessageW(hWnd, WM_THEMECHANGED, 0, 0);
 			}
+			SetAssertDialogParent(hWnd);
 			break;
 
 		case WM_USER_RUN_CALLBACK:
@@ -839,12 +933,18 @@ namespace MainWindow
 				} else {
 					g_activeWindow = WINDOW_OTHER;
 				}
+
 				if (!noFocusPause && g_Config.bPauseOnLostFocus && GetUIState() == UISTATE_INGAME) {
 					if (pause != Core_IsStepping()) {
-						if (disasmWindow)
+						if (disasmWindow) {
 							SendMessage(disasmWindow->GetDlgHandle(), WM_COMMAND, IDC_STOPGO, 0);
-						else
-							Core_EnableStepping(pause, "ui.lost_focus", 0);
+						} else {
+							if (pause) {
+								Core_Break(BreakReason::UIFocus, 0);
+							} else if (Core_BreakReason() == BreakReason::UIFocus) {
+								Core_Resume();
+							}
+						}
 					}
 				}
 
@@ -899,7 +999,7 @@ namespace MainWindow
 				break;
 
 			case SIZE_MINIMIZED:
-				Core_NotifyWindowHidden(true);
+				Native_NotifyWindowHidden(true);
 				if (!g_Config.bPauseWhenMinimized) {
 					System_PostUIMessage(UIMessage::WINDOW_MINIMIZED, "true");
 				}
@@ -1018,7 +1118,6 @@ namespace MainWindow
 					if (DragQueryFile(hdrop, 0, filename, ARRAY_SIZE(filename)) != 0) {
 						const std::string utf8_filename = ReplaceAll(ConvertWStringToUTF8(filename), "\\", "/");
 						System_PostUIMessage(UIMessage::REQUEST_GAME_BOOT, utf8_filename);
-						Core_EnableStepping(false);
 					}
 				}
 				DragFinish(hdrop);
@@ -1026,12 +1125,17 @@ namespace MainWindow
 			break;
 
 		case WM_CLOSE:
+		{
+			if (ConfirmAction(hWnd, false)) {
+				DestroyWindow(hWnd);
+			}
+			return 0;
+		}
+
+		case WM_DESTROY:
 			InputDevice::StopPolling();
 			MainThread_Stop();
 			WindowsRawInput::Shutdown();
-			return DefWindowProc(hWnd,message,wParam,lParam);
-
-		case WM_DESTROY:
 			KillTimer(hWnd, TIMER_CURSORUPDATE);
 			KillTimer(hWnd, TIMER_CURSORMOVEUPDATE);
 			// Main window is gone, this tells the message loop to exit.
@@ -1062,7 +1166,6 @@ namespace MainWindow
 			NativeSetRestarting();
 			InputDevice::StopPolling();
 			MainThread_Stop();
-			coreState = CORE_POWERUP;
 			UpdateUIState(UISTATE_MENU);
 			MainThread_Start(g_Config.iGPUBackend == (int)GPUBackend::OPENGL);
 			InputDevice::BeginPolling();
@@ -1072,6 +1175,10 @@ namespace MainWindow
 			UpdateSwitchUMD();
 			break;
 
+		case WM_USER_DESTROY:
+			DestroyWindow(hWnd);
+			break;
+
 		case WM_MENUSELECT:
 			// Called when a menu is opened. Also when an item is selected, but meh.
 			UpdateMenus(true);
@@ -1143,11 +1250,11 @@ namespace MainWindow
 
 	void ToggleDebugConsoleVisibility() {
 		if (!g_Config.bEnableLogging) {
-			LogManager::GetInstance()->GetConsoleListener()->Show(false);
+			g_logManager.GetConsoleListener()->Show(false);
 			EnableMenuItem(menu, ID_DEBUG_LOG, MF_GRAYED);
 		}
 		else {
-			LogManager::GetInstance()->GetConsoleListener()->Show(true);
+			g_logManager.GetConsoleListener()->Show(true);
 			EnableMenuItem(menu, ID_DEBUG_LOG, MF_ENABLED);
 		}
 	}
