@@ -97,8 +97,6 @@ public abstract class NativeActivity extends Activity {
 
 	private Vibrator vibrator;
 
-	private boolean isXperiaPlay;
-
 	// This is to avoid losing the game/menu state etc when we are just
 	// switched-away from or rotated etc.
 	private boolean shuttingDown;
@@ -136,6 +134,15 @@ public abstract class NativeActivity extends Activity {
 	public static final int REQUEST_CODE_CAMERA_PERMISSION = 3;
 	public static final int REQUEST_CODE_MICROPHONE_PERMISSION = 4;
 
+	// Once we received a "modern" mouse event, we stop listening to old style mouse
+	// button events.
+	public static boolean useModernMouseEvents = false;
+
+	// Workaround for bizarre behavior on Pocophone where we get modern events
+	// for the left mouse button, but wacky BACK keyboard event with source == mouse
+	// for the right mouse button.
+	public static boolean useModernMouseEventsB2 = false;
+
 	// Functions for the app activity to override to change behaviour.
 
 	public native void registerCallbacks();
@@ -275,7 +282,7 @@ public abstract class NativeActivity extends Activity {
 			for (String var : varNames) {
 				Log.i(TAG, "getSdCardPaths: Checking env " + var);
 				String secStore = System.getenv("SECONDARY_STORAGE");
-				if (secStore != null && secStore.length() > 0) {
+				if (secStore != null && !secStore.isEmpty()) {
 					list = new ArrayList<String>();
 					list.add(secStore);
 					break;
@@ -399,8 +406,6 @@ public abstract class NativeActivity extends Activity {
 		// All other device types are treated the same.
 		}
 
-		isXperiaPlay = IsXperiaPlay();
-
 		String extStorageState = Environment.getExternalStorageState();
 		String extStorageDir = Environment.getExternalStorageDirectory().getAbsolutePath();
 		File externalFiles = this.getExternalFilesDir(null);
@@ -573,10 +578,8 @@ public abstract class NativeActivity extends Activity {
 	// Need API 11 to check for existence of a vibrator? Zany.
 	@TargetApi(Build.VERSION_CODES.HONEYCOMB)
 	public void checkForVibrator() {
-		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
-			if (!vibrator.hasVibrator()) {
-				vibrator = null;
-			}
+		if (!vibrator.hasVibrator()) {
+			vibrator = null;
 		}
 	}
 
@@ -636,20 +639,14 @@ public abstract class NativeActivity extends Activity {
 			// the format here, if we specify that we want destination alpha in the config chooser, which we do.
 			// http://grokbase.com/t/gg/android-developers/11bj40jm4w/fall-back
 
-			// Needed to avoid banding on Ouya?
-			if (Build.MANUFACTURER.equals("OUYA")) {
-				mGLSurfaceView.getHolder().setFormat(PixelFormat.RGBX_8888);
-				mGLSurfaceView.setEGLConfigChooser(new NativeEGLConfigChooser());
-			} else {
-				// Tried to mess around with config choosers (NativeEGLConfigChooser) here but fail completely on Xperia Play.
+			// Tried to mess around with config choosers (NativeEGLConfigChooser) here but fail completely on Xperia Play.
 
-				// Then I tried to require 8888/16/8 but that backfired too, does not work on Mali 450 which is
-				// used in popular TVs and boxes like Mi Box. So we'll just get what we get, I guess...
+			// Then I tried to require 8888/16/8 but that backfired too, does not work on Mali 450 which is
+			// used in popular TVs and boxes like Mi Box. So we'll just get what we get, I guess...
 
-				// if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH && !Build.MANUFACTURER.equals("Amazon")) {
-					// mGLSurfaceView.setEGLConfigChooser(8, 8, 8, 8, 16, 8);
-				// }
-			}
+			// if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH && !Build.MANUFACTURER.equals("Amazon")) {
+				// mGLSurfaceView.setEGLConfigChooser(8, 8, 8, 8, 16, 8);
+			// }
 
 			mGLSurfaceView.setRenderer(nativeRenderer);
 			setContentView(mGLSurfaceView);
@@ -786,7 +783,7 @@ public abstract class NativeActivity extends Activity {
 					do {
 						try {
 							Thread.sleep(10);
-						} catch (InterruptedException e) {
+						} catch (InterruptedException ignored) {
 						}
 						tries--;
 					} while (nativeRenderer.isRenderingFrame() && tries > 0);
@@ -839,7 +836,7 @@ public abstract class NativeActivity extends Activity {
 		super.onPause();
 		Log.i(TAG, "onPause");
 		loseAudioFocus(this.audioManager, this.audioFocusChangeListener);
-		sizeManager.setPaused(true);
+		sizeManager.onPause();
 		NativeApp.pause();
 		if (!javaGL) {
 			mSurfaceView.onPause();
@@ -875,7 +872,7 @@ public abstract class NativeActivity extends Activity {
 	protected void onResume() {
 		super.onResume();
 		updateSustainedPerformanceMode();
-		sizeManager.setPaused(false);
+		sizeManager.onResume();
 		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
 			updateSystemUiVisibility();
 		}
@@ -979,8 +976,8 @@ public abstract class NativeActivity extends Activity {
 			for (InputDeviceState input : inputPlayers) {
 				buffer += input.getDebugString();
 			}
-			if (buffer.length() == 0) {
-				buffer = "(no devices)";
+			if (buffer.isEmpty()) {
+				return "(no devices)";
 			}
 			return buffer;
 		} else {
@@ -988,15 +985,29 @@ public abstract class NativeActivity extends Activity {
 		}
 	}
 
-	public boolean IsXperiaPlay() {
-		return android.os.Build.MODEL.equals("R800a") || android.os.Build.MODEL.equals("R800i") || android.os.Build.MODEL.equals("R800x") || android.os.Build.MODEL.equals("R800at") || android.os.Build.MODEL.equals("SO-01D") || android.os.Build.MODEL.equals("zeus");
-	}
-
 	// We grab the keys before onKeyDown/... even see them. This is also better because it lets us
 	// distinguish devices.
 	@Override
 	public boolean dispatchKeyEvent(KeyEvent event) {
-		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR1 && !isXperiaPlay) {
+		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR1) {
+			// Log.d(TAG, "key event source: " + event.getSource());
+			if (NativeSurfaceView.isFromSource(event, InputDevice.SOURCE_MOUSE)) {
+				Log.i(TAG, "Forwarding key event from mouse: " + event.getKeyCode());
+				Log.i(TAG, "usemodernb2: " + useModernMouseEventsB2);
+				if (event.getKeyCode() == KeyEvent.KEYCODE_BACK && !useModernMouseEventsB2) {
+					// Probably a right click
+					switch (event.getAction()) {
+						case KeyEvent.ACTION_DOWN:
+							NativeApp.mouse(-1, -1, 2, 1);
+							break;
+						case KeyEvent.ACTION_UP:
+							NativeApp.mouse(-1, -1, 2, 2);
+							break;
+					}
+				}
+				return true;
+			}
+
 			InputDeviceState state = getInputDeviceState(event);
 			if (state == null) {
 				return super.dispatchKeyEvent(event);
@@ -1028,12 +1039,14 @@ public abstract class NativeActivity extends Activity {
 			if (!passThrough) {
 				switch (event.getAction()) {
 				case KeyEvent.ACTION_DOWN:
+					Log.i(TAG, "KeyEvent Down");
 					if (state.onKeyDown(event)) {
 						return true;
 					}
 					break;
 
 				case KeyEvent.ACTION_UP:
+					Log.i(TAG, "KeyEvent Up");
 					if (state.onKeyUp(event)) {
 						return true;
 					}
@@ -1061,23 +1074,24 @@ public abstract class NativeActivity extends Activity {
 
 	@TargetApi(Build.VERSION_CODES.N)
 	void sendMouseDelta(float dx, float dy) {
-		NativeApp.mouseDelta(dx, dy);
+		// Ignore zero deltas.
+		if (Math.abs(dx) > 0.001 || Math.abs(dx) > 0.001) {
+			NativeApp.mouseDelta(dx, dy);
+		}
 	}
 
 	@Override
 	@TargetApi(Build.VERSION_CODES.HONEYCOMB_MR1)
 	public boolean onGenericMotionEvent(MotionEvent event) {
-		// Log.d(TAG, "onGenericMotionEvent: " + event);
+		// Log.i(TAG, "NativeActivity onGenericMotionEvent: " + event);
 		if (InputDeviceState.inputSourceIsJoystick(event.getSource())) {
-			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR1) {
-				InputDeviceState state = getInputDeviceState(event);
-				if (state == null) {
-					Log.w(TAG, "Joystick event but failed to get input device state.");
-					return super.onGenericMotionEvent(event);
-				}
-				state.onJoystickMotion(event);
-				return true;
+			InputDeviceState state = getInputDeviceState(event);
+			if (state == null) {
+				Log.w(TAG, "Joystick event but failed to get input device state.");
+				return super.onGenericMotionEvent(event);
 			}
+			state.onJoystickMotion(event);
+			return true;
 		}
 
 		if ((event.getSource() & InputDevice.SOURCE_CLASS_POINTER) != 0) {
@@ -1087,15 +1101,44 @@ public abstract class NativeActivity extends Activity {
 					float dy = event.getAxisValue(MotionEvent.AXIS_RELATIVE_Y);
 					sendMouseDelta(dx, dy);
 				}
+				switch (event.getAction()) {
+					case MotionEvent.ACTION_MOVE:
+						Log.i(TAG, "Erroneous move event"); // should be in touch events
+						return true;
+					case MotionEvent.ACTION_HOVER_MOVE:
+						// Log.i(TAG, "Action Hover Move");
+						// process the mouse hover movement...
+						NativeApp.mouse(event.getX(), event.getY(), 0, 0);
+						return true;
+					case MotionEvent.ACTION_SCROLL:
+						float scrollX = event.getAxisValue(MotionEvent.AXIS_HSCROLL);
+						float scrollY = event.getAxisValue(MotionEvent.AXIS_VSCROLL);
+						// Log.i(TAG, "Action Scroll: " + scrollX + " " + scrollY);
+						NativeApp.mouseWheelEvent(scrollX, scrollY);
+						return true;
+				}
 			}
+		}
 
-			switch (event.getAction()) {
-			case MotionEvent.ACTION_HOVER_MOVE:
-				// process the mouse hover movement...
-				return true;
-			case MotionEvent.ACTION_SCROLL:
-				NativeApp.mouseWheelEvent(event.getAxisValue(MotionEvent.AXIS_HSCROLL), event.getAxisValue(MotionEvent.AXIS_VSCROLL));
-				return true;
+		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
+			int button = event.getActionButton();
+			switch (event.getActionMasked()) {
+				case MotionEvent.ACTION_BUTTON_PRESS: {
+					Log.i(TAG, "action button press: button: " + button);
+					useModernMouseEvents = true;
+					if (button > 1) {
+						useModernMouseEventsB2 = true;
+					}
+					NativeApp.mouse(event.getX(), event.getY(), button, 1);
+					return true;
+				}
+				case MotionEvent.ACTION_BUTTON_RELEASE: {
+					Log.i(TAG, "action button release: button: " + button);
+					NativeApp.mouse(event.getX(), event.getY(), button, 2);
+					return true;
+				}
+				default:
+					break;
 			}
 		}
 		return super.onGenericMotionEvent(event);
