@@ -1,3 +1,4 @@
+
 // Headless version of PPSSPP, for testing using http://code.google.com/p/pspautotests/ .
 // See headless.txt.
 // To build on non-windows systems, just run CMake in the SDL directory, it will build both a normal ppsspp and the headless version.
@@ -12,6 +13,8 @@
 #include <jni.h>
 #endif
 
+#include <algorithm>
+
 #include "Common/Profiler/Profiler.h"
 #include "Common/System/NativeApp.h"
 #include "Common/System/Request.h"
@@ -30,6 +33,7 @@
 #include "Common/File/FileUtil.h"
 #include "Common/GraphicsContext.h"
 #include "Common/TimeUtil.h"
+#include "Common/StringUtils.h"
 #include "Common/Thread/ThreadManager.h"
 #include "Core/Config.h"
 #include "Core/ConfigValues.h"
@@ -66,33 +70,6 @@ bool System_AudioRecordingIsAvailable() { return false; }
 bool System_AudioRecordingState() { return false; }
 #endif
 
-class PrintfLogger : public LogListener {
-public:
-	void Log(const LogMessage &message) override {
-		switch (message.level) {
-		case LogLevel::LVERBOSE:
-			fprintf(stderr, "V %s", message.msg.c_str());
-			break;
-		case LogLevel::LDEBUG:
-			fprintf(stderr, "D %s", message.msg.c_str());
-			break;
-		case LogLevel::LINFO:
-			fprintf(stderr, "I %s", message.msg.c_str());
-			break;
-		case LogLevel::LERROR:
-			fprintf(stderr, "E %s", message.msg.c_str());
-			break;
-		case LogLevel::LWARNING:
-			fprintf(stderr, "W %s", message.msg.c_str());
-			break;
-		case LogLevel::LNOTICE:
-		default:
-			fprintf(stderr, "N %s", message.msg.c_str());
-			break;
-		}
-	}
-};
-
 // Temporary hacks around annoying linking errors.
 void NativeFrame(GraphicsContext *graphicsContext) { }
 void NativeResized() { }
@@ -117,6 +94,7 @@ bool System_GetPropertyBool(SystemProperty prop) {
 }
 void System_Notify(SystemNotification notification) {}
 void System_PostUIMessage(UIMessage message, const std::string &param) {}
+void System_RunOnMainThread(std::function<void()>) {}
 bool System_MakeRequest(SystemRequestType type, int requestId, const std::string &param1, const std::string &param2, int64_t param3, int64_t param4) {
 	switch (type) {
 	case SystemRequestType::SEND_DEBUG_OUTPUT:
@@ -139,7 +117,7 @@ void System_AskForPermission(SystemPermission permission) {}
 PermissionStatus System_GetPermissionStatus(SystemPermission permission) { return PERMISSION_STATUS_GRANTED; }
 void System_AudioGetDebugStats(char *buf, size_t bufSize) { if (buf) buf[0] = '\0'; }
 void System_AudioClear() {}
-void System_AudioPushSamples(const s32 *audio, int numSamples) {}
+void System_AudioPushSamples(const s32 *audio, int numSamples, float volume) {}
 
 // TODO: To avoid having to define these here, these should probably be turned into system "requests".
 bool NativeSaveSecret(std::string_view nameOfSecret, std::string_view data) { return false; }
@@ -207,9 +185,9 @@ bool RunAutoTest(HeadlessHost *headlessHost, CoreParameter &coreParameter, const
 	if (opt.compare || opt.bench)
 		coreParameter.collectDebugOutput = &output;
 
-	std::string error_string;
-	if (!PSP_InitStart(coreParameter, &error_string)) {
-		fprintf(stderr, "Failed to start '%s'. Error: %s\n", coreParameter.fileToStart.c_str(), error_string.c_str());
+	if (!PSP_InitStart(coreParameter)) {
+		// Shouldn't really happen anymore, the errors happen later in PSP_InitUpdate.
+		fprintf(stderr, "Failed to start '%s'.\n", coreParameter.fileToStart.c_str());
 		printf("TESTERROR\n");
 		TeamCityPrint("testIgnored name='%s' message='PRX/ELF missing'", currentTestName.c_str());
 		GitHubActionsPrint("error", "PRX/ELF missing for %s", currentTestName.c_str());
@@ -221,9 +199,12 @@ bool RunAutoTest(HeadlessHost *headlessHost, CoreParameter &coreParameter, const
 	if (opt.compare)
 		headlessHost->SetComparisonScreenshot(ExpectedScreenshotFromFilename(coreParameter.fileToStart), opt.maxScreenshotError);
 
-	while (!PSP_InitUpdate(&error_string))
-		sleep_ms(1);
+	std::string error_string;
+	while (PSP_InitUpdate(&error_string) == BootState::Booting)
+		sleep_ms(1, "auto-test");
+
 	if (!PSP_IsInited()) {
+		TeamCityPrint("%s", error_string.c_str());
 		TeamCityPrint("testFailed name='%s' message='Startup failed'", currentTestName.c_str());
 		TeamCityPrint("testFinished name='%s'", currentTestName.c_str());
 		GitHubActionsPrint("error", "Test init failed for %s", currentTestName.c_str());
@@ -232,30 +213,38 @@ bool RunAutoTest(HeadlessHost *headlessHost, CoreParameter &coreParameter, const
 
 	System_Notify(SystemNotification::BOOT_DONE);
 
-	Core_UpdateDebugStats((DebugOverlay)g_Config.iDebugOverlay == DebugOverlay::DEBUG_STATS || g_Config.bLogFrameDrops);
+	PSP_UpdateDebugStats((DebugOverlay)g_Config.iDebugOverlay == DebugOverlay::DEBUG_STATS || g_Config.bLogFrameDrops);
 
-	PSP_BeginHostFrame();
+	if (gpu) {
+		gpu->BeginHostFrame();
+	}
 	Draw::DrawContext *draw = coreParameter.graphicsContext ? coreParameter.graphicsContext->GetDrawContext() : nullptr;
-	if (draw)
+	if (draw) {
 		draw->BeginFrame(Draw::DebugFlags::NONE);
+	}
 
 	bool passed = true;
 	double deadline = time_now_d() + opt.timeout;
-	coreState = coreParameter.startBreak ? CORE_STEPPING : CORE_RUNNING;
-	while (coreState == CORE_RUNNING || coreState == CORE_STEPPING)
+	coreState = coreParameter.startBreak ? CORE_STEPPING_CPU : CORE_RUNNING_CPU;
+	while (coreState == CORE_RUNNING_CPU || coreState == CORE_STEPPING_CPU)
 	{
 		int blockTicks = (int)usToCycles(1000000 / 10);
 		PSP_RunLoopFor(blockTicks);
 
 		// If we were rendering, this might be a nice time to do something about it.
 		if (coreState == CORE_NEXTFRAME) {
-			coreState = CORE_RUNNING;
+			coreState = CORE_RUNNING_CPU;
 			headlessHost->SwapBuffers();
 		}
-		if (coreState == CORE_STEPPING && !coreParameter.startBreak) {
+		if (coreState == CORE_STEPPING_CPU && !coreParameter.startBreak) {
 			break;
 		}
-		if (time_now_d() > deadline) {
+		bool debugger = false;
+#ifdef _WIN32
+		if (IsDebuggerPresent())
+			debugger = true;
+#endif
+		if (time_now_d() > deadline && !debugger) {
 			// Don't compare, print the output at least up to this point, and bail.
 			if (!opt.bench) {
 				printf("%s", output.c_str());
@@ -269,7 +258,9 @@ bool RunAutoTest(HeadlessHost *headlessHost, CoreParameter &coreParameter, const
 			Core_Stop();
 		}
 	}
-	PSP_EndHostFrame();
+	if (gpu) {
+		gpu->EndHostFrame();
+	}
 
 	if (draw) {
 		draw->BindFramebufferAsRenderTarget(nullptr, { Draw::RPAction::CLEAR, Draw::RPAction::DONT_CARE, Draw::RPAction::DONT_CARE }, "Headless");
@@ -280,7 +271,7 @@ bool RunAutoTest(HeadlessHost *headlessHost, CoreParameter &coreParameter, const
 		draw->EndFrame();
 	}
 
-	PSP_Shutdown();
+	PSP_Shutdown(true);
 
 	if (!opt.bench)
 		headlessHost->FlushDebugOutput();
@@ -315,12 +306,42 @@ std::vector<std::string> ReadFromListFile(const std::string &listFilename) {
 	return testFilenames;
 }
 
+static void AddRecursively(std::vector<std::string> *tests, Path actualPath) {
+	// TODO: Some file systems can optimize this.
+	std::vector<File::FileInfo> fileInfo;
+	if (!File::GetFilesInDir(actualPath, &fileInfo, "prx")) {
+		return;
+	}
+	for (const auto &file : fileInfo) {
+		if (file.isDirectory) {
+			AddRecursively(tests, actualPath / file.name);
+		} else if (file.name != "Makefile") {  // hack around filter problem
+			tests->push_back((actualPath / file.name).ToString());
+		}
+	}
+}
+
+static void AddTestsByPath(std::vector<std::string> *tests, std::string_view path) {
+	if (endsWith(path, "/...")) {
+		path = path.substr(0, path.size() - 4);
+		// Recurse for tests
+		AddRecursively(tests, Path(path));
+	} /* else if (File::IsDirectory(Path(path))) {
+		// Alternate syntax - just specify the path.
+		AddRecursively(tests, Path(path));
+	} */ else {
+		tests->push_back(std::string(path));
+	}
+}
+
 int main(int argc, const char* argv[])
 {
 	PROFILE_INIT();
 	TimeInit();
 #if PPSSPP_PLATFORM(WINDOWS)
-	SetCleanExitOnAssert();
+	if (!IsDebuggerPresent()) {
+		SetCleanExitOnAssert();
+	}
 #else
 	// Ignore sigpipe.
 	if (signal(SIGPIPE, SIG_IGN) == SIG_ERR) {
@@ -339,9 +360,11 @@ int main(int argc, const char* argv[])
 	GPUCore gpuCore = GPUCORE_SOFTWARE;
 	CPUCore cpuCore = CPUCore::JIT;
 	int debuggerPort = -1;
-	bool newAtrac = false;
+	bool oldAtrac = false;
+	bool outputDebugStringLog = false;
 
 	std::vector<std::string> testFilenames;
+	std::vector<std::string> ignoredTests;
 	const char *mountIso = nullptr;
 	const char *mountRoot = nullptr;
 	const char *screenshotFilename = nullptr;
@@ -362,6 +385,8 @@ int main(int argc, const char* argv[])
 		}
 		else if (!strcmp(argv[i], "-l") || !strcmp(argv[i], "--log"))
 			fullLog = true;
+		else if (!strcmp(argv[i], "-o") || !strcmp(argv[i], "--odslog"))
+			outputDebugStringLog = true;
 		else if (!strcmp(argv[i], "-i"))
 			cpuCore = CPUCore::INTERPRETER;
 		else if (!strcmp(argv[i], "-j"))
@@ -376,8 +401,8 @@ int main(int argc, const char* argv[])
 			testOptions.bench = true;
 		else if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "--verbose"))
 			testOptions.verbose = true;
-		else if (!strcmp(argv[i], "--new-atrac"))
-			newAtrac = true;
+		else if (!strcmp(argv[i], "--old-atrac"))
+			oldAtrac = true;
 		else if (!strncmp(argv[i], "--graphics=", strlen("--graphics=")) && strlen(argv[i]) > strlen("--graphics="))
 		{
 			const char *gpuName = argv[i] + strlen("--graphics=");
@@ -416,27 +441,43 @@ int main(int argc, const char* argv[])
 			stateToLoad = argv[i] + strlen("--state=");
 		else if (!strcmp(argv[i], "--help") || !strcmp(argv[i], "-h"))
 			return printUsage(argv[0], NULL);
-		else
-			testFilenames.push_back(argv[i]);
+		else if (!strcmp(argv[i], "--ignore")) {
+			if (++i >= argc)
+				return printUsage(argv[0], "Missing argument after --ignore");
+			ignoredTests.push_back(argv[i]);
+		} else {
+			AddTestsByPath(&testFilenames, argv[i]);
+		}
 	}
 
 	if (testFilenames.size() == 1 && testFilenames[0][0] == '@')
 		testFilenames = ReadFromListFile(testFilenames[0].substr(1));
 
+	// Remove any ignored tests.
+	testFilenames.erase(
+		std::remove_if(
+			testFilenames.begin(),
+			testFilenames.end(),
+			[&ignoredTests](const std::string& item) { return std::find(ignoredTests.begin(), ignoredTests.end(), item) != ignoredTests.end(); }
+		),
+		testFilenames.end()
+	);
+
 	if (testFilenames.empty())
 		return printUsage(argv[0], argc <= 1 ? NULL : "No executables specified");
 
-	LogManager::Init(&g_Config.bEnableLogging);
-	LogManager *logman = LogManager::GetInstance();
-
-	PrintfLogger *printfLogger = new PrintfLogger();
+	g_Config.bEnableLogging = (fullLog || outputDebugStringLog);
+	g_logManager.Init(&g_Config.bEnableLogging, outputDebugStringLog);
 
 	for (int i = 0; i < (int)Log::NUMBER_OF_LOGS; i++) {
 		Log type = (Log)i;
-		logman->SetEnabled(type, fullLog);
-		logman->SetLogLevel(type, LogLevel::LDEBUG);
+		g_logManager.SetEnabled(type, (fullLog || outputDebugStringLog));
+		g_logManager.SetLogLevel(type, LogLevel::LDEBUG);
+	}
+	if (fullLog) {
+		// Only with --log, add the printfLogger.
+		g_logManager.EnableOutput(LogOutput::Printf);
 	}
-	logman->AddListener(printfLogger);
 
 	// Needs to be after log so we don't interfere with test output.
 	g_threadManager.Init(cpu_info.num_cores, cpu_info.logical_cpu_count);
@@ -485,7 +526,7 @@ int main(int argc, const char* argv[])
 	g_Config.iLockParentalLevel = 9;
 	g_Config.iInternalResolution = 1;
 	g_Config.iFastForwardMode = (int)FastForwardMode::CONTINUOUS;
-	g_Config.bEnableLogging = fullLog;
+	g_Config.bEnableLogging = (fullLog || outputDebugStringLog);
 	g_Config.bSoftwareSkinning = true;
 	g_Config.bVertexDecoderJit = true;
 	g_Config.bSoftwareRendering = coreParameter.gpuCore == GPUCORE_SOFTWARE;
@@ -498,10 +539,11 @@ int main(int argc, const char* argv[])
 	g_Config.sMACAddress = "12:34:56:78:9A:BC";
 	g_Config.iFirmwareVersion = PSP_DEFAULT_FIRMWARE;
 	g_Config.iPSPModel = PSP_MODEL_SLIM;
-	g_Config.iGlobalVolume = VOLUME_FULL;
-	g_Config.iReverbVolume = VOLUME_FULL;
+	g_Config.iGameVolume = VOLUMEHI_FULL;
+	g_Config.iReverbVolume = VOLUMEHI_FULL;
 	g_Config.internalDataDirectory.clear();
-	g_Config.bUseExperimentalAtrac = newAtrac;
+	g_Config.bUseOldAtrac = oldAtrac;
+	g_Config.iForceEnableHLE = 0xFFFFFFFF;  // Run all modules as HLE. We don't have anything to load in this context.
 
 	Path exePath = File::GetExeDirectory();
 	g_Config.flash0Directory = exePath / "assets/flash0";
@@ -618,8 +660,7 @@ int main(int argc, const char* argv[])
 	g_headlessHost = nullptr;
 
 	g_VFS.Clear();
-	LogManager::Shutdown();
-	delete printfLogger;
+	g_logManager.Shutdown();
 
 #if PPSSPP_PLATFORM(WINDOWS)
 	timeEndPeriod(1);
