@@ -16,7 +16,6 @@
 #include "Common/TimeUtil.h"
 #include "Common/File/FileUtil.h"
 #include "Common/Serialize/Serializer.h"
-#include "Common/Log/StdioListener.h"
 #include "Common/Input/InputState.h"
 #include "Common/Thread/ThreadUtil.h"
 #include "Common/Thread/ThreadManager.h"
@@ -40,7 +39,7 @@
 #include "Core/ELF/ParamSFO.h"
 
 #include "GPU/GPUState.h"
-#include "GPU/GPUInterface.h"
+#include "GPU/GPUCommon.h"
 #include "GPU/Common/FramebufferManagerCommon.h"
 #include "GPU/Common/TextureScalerCommon.h"
 #include "GPU/Common/PresentationCommon.h"
@@ -90,7 +89,6 @@ static struct {
 #define VSYNC_SWAP_INTERVAL_RUN_SPEED_THRESHOLD 5.0f
 
 static bool libretro_supports_bitmasks = false;
-static bool libretro_supports_option_categories = false;
 static bool show_ip_address_options = true;
 static bool show_upnp_port_option = true;
 static bool show_detect_frame_rate_option = true;
@@ -106,6 +104,9 @@ namespace Libretro
    static retro_input_state_t input_state_cb;
    static retro_log_printf_t log_cb;
 
+   bool g_pendingBoot = false;
+   std::string g_bootErrorString;
+
    static bool detectVsyncSwapInterval = false;
    static bool detectVsyncSwapIntervalOptShown = true;
    static bool softwareRenderInitHack = false;
@@ -293,41 +294,27 @@ namespace Libretro
 
 using namespace Libretro;
 
-class PrintfLogger : public LogListener
-{
-   public:
-      PrintfLogger(retro_log_callback log) : log_(log.log) {}
-      void Log(const LogMessage &message)
-      {
-         switch (message.level)
-         {
-            case LogLevel::LVERBOSE:
-            case LogLevel::LDEBUG:
-               log_(RETRO_LOG_DEBUG, "[%s] %s",
-                     message.log, message.msg.c_str());
-               break;
+void RetroLogCallback(const LogMessage &message, void *userdata) {
+   retro_log_printf_t fn = (retro_log_printf_t)userdata;
+   switch (message.level) {
+   case LogLevel::LVERBOSE:
+   case LogLevel::LDEBUG:
+      (fn)(RETRO_LOG_DEBUG, "[%s] %s", message.log, message.msg.c_str());
+      break;
 
-            case LogLevel::LERROR:
-               log_(RETRO_LOG_ERROR, "[%s] %s",
-                     message.log, message.msg.c_str());
-               break;
-            case LogLevel::LNOTICE:
-            case LogLevel::LWARNING:
-               log_(RETRO_LOG_WARN, "[%s] %s",
-                     message.log, message.msg.c_str());
-               break;
-            case LogLevel::LINFO:
-            default:
-               log_(RETRO_LOG_INFO, "[%s] %s",
-                     message.log, message.msg.c_str());
-               break;
-         }
-      }
-
-   private:
-      retro_log_printf_t log_;
-};
-static PrintfLogger *printfLogger;
+   case LogLevel::LERROR:
+      (fn)(RETRO_LOG_ERROR, "[%s] %s", message.log, message.msg.c_str());
+      break;
+   case LogLevel::LNOTICE:
+   case LogLevel::LWARNING:
+      (fn)(RETRO_LOG_WARN, "[%s] %s", message.log, message.msg.c_str());
+      break;
+   case LogLevel::LINFO:
+   default:
+      (fn)(RETRO_LOG_INFO, "[%s] %s", message.log, message.msg.c_str());
+      break;
+   }
+}
 
 static bool set_variable_visibility(void)
 {
@@ -405,7 +392,8 @@ void retro_set_environment(retro_environment_t cb)
 {
    environ_cb = cb;
 
-   libretro_set_core_options(environ_cb, &libretro_supports_option_categories);
+   bool option_categories = false;
+   libretro_set_core_options(environ_cb, &option_categories);
    struct retro_core_options_update_display_callback update_display_cb;
    update_display_cb.callback = set_variable_visibility;
    environ_cb(RETRO_ENVIRONMENT_SET_CORE_OPTIONS_UPDATE_DISPLAY_CALLBACK, &update_display_cb);
@@ -494,13 +482,6 @@ static void check_variables(CoreParameter &coreParam)
           coreParam.fastForward = isFastForwarding;
    }
 
-   bool updated = false;
-
-   if (     coreState != CoreState::CORE_POWERUP
-         && environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE_UPDATE, &updated)
-         && !updated)
-      return;
-
    struct retro_variable var = {0};
    std::string sTextureShaderName_prev;
    int iInternalResolution_prev;
@@ -715,7 +696,6 @@ static void check_variables(CoreParameter &coreParam)
          g_Config.iInternalResolution = 1;
    }
 
-#if 0 // see issue #16786
    var.key = "ppsspp_mulitsample_level";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    {
@@ -730,7 +710,6 @@ static void check_variables(CoreParameter &coreParam)
       else if (!strcmp(var.value, "x8"))
          g_Config.iMultiSampleLevel = 3;
    }
-#endif
 
    var.key = "ppsspp_cropto16x9";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
@@ -787,11 +766,11 @@ static void check_variables(CoreParameter &coreParam)
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    {
       if (!strcmp(var.value, "No buffer"))
-         g_Config.iInflightFrames = 0;
-      else if (!strcmp(var.value, "Up to 1"))
          g_Config.iInflightFrames = 1;
-      else if (!strcmp(var.value, "Up to 2"))
+      else if (!strcmp(var.value, "Up to 1"))
          g_Config.iInflightFrames = 2;
+      else if (!strcmp(var.value, "Up to 2"))
+         g_Config.iInflightFrames = 3;
    }
 
    var.key = "ppsspp_skip_buffer_effects";
@@ -1158,7 +1137,6 @@ static void check_variables(CoreParameter &coreParam)
          gpu->NotifyDisplayResized();
    }
 
-#if 0 // see issue #16786
    if (g_Config.iMultiSampleLevel != iMultiSampleLevel_prev && PSP_IsInited())
    {
       if (gpu)
@@ -1166,7 +1144,6 @@ static void check_variables(CoreParameter &coreParam)
          gpu->NotifyRenderResized();
       }
    }
-#endif
 
    if (updateAvInfo)
    {
@@ -1204,18 +1181,15 @@ static const struct retro_controller_info ports[] =
 void retro_init(void)
 {
    TimeInit();
+   SetCurrentThreadName("Main");
 
    struct retro_log_callback log;
    if (environ_cb(RETRO_ENVIRONMENT_GET_LOG_INTERFACE, &log))
    {
       log_cb = log.log;
-      LogManager::Init(&g_Config.bEnableLogging);
-      printfLogger = new PrintfLogger(log);
-      LogManager* logman = LogManager::GetInstance();
-      logman->RemoveListener(logman->GetStdioListener());
-      logman->RemoveListener(logman->GetDebuggerListener());
-      logman->ChangeFileLog(nullptr);
-      logman->AddListener(printfLogger);
+      g_logManager.Init(&g_Config.bEnableLogging);
+      g_logManager.SetOutputsEnabled(LogOutput::ExternalCallback);
+      g_logManager.SetExternalLogCallback(&RetroLogCallback, (void *)log_cb);
    }
 
    VsyncSwapIntervalReset();
@@ -1249,7 +1223,7 @@ void retro_init(void)
    g_Config.iInternalResolution = 0;
 
    // Log levels must be set after g_Config.Load
-   LogManager::GetInstance()->SetAllLogLevels(LogLevel::LINFO);
+   g_logManager.SetAllLogLevels(LogLevel::LINFO);
 
    const char* nickname = NULL;
    if (environ_cb(RETRO_ENVIRONMENT_GET_USERNAME, &nickname) && nickname)
@@ -1284,14 +1258,10 @@ void retro_init(void)
 void retro_deinit(void)
 {
    g_threadManager.Teardown();
-   LogManager::Shutdown();
+   g_logManager.Shutdown();
    log_cb = NULL;
 
-   delete printfLogger;
-   printfLogger = nullptr;
-
    libretro_supports_bitmasks = false;
-   libretro_supports_option_categories = false;
 
    VsyncSwapIntervalReset();
 
@@ -1319,6 +1289,8 @@ void retro_get_system_av_info(struct retro_system_av_info *info)
    info->timing.fps            = (60.0 / 1.001) / (double)vsyncSwapInterval;
    info->timing.sample_rate    = SAMPLERATE;
 
+   _dbg_assert_(g_Config.iInternalResolution != 0);
+
    info->geometry.base_width   = g_Config.iInternalResolution * NATIVEWIDTH;
    info->geometry.base_height  = g_Config.iInternalResolution * NATIVEHEIGHT;
    info->geometry.max_width    = g_Config.iInternalResolution * NATIVEWIDTH;
@@ -1349,15 +1321,27 @@ namespace Libretro
    static void EmuFrame()
    {
       ctx->SetRenderTarget();
-      if (ctx->GetDrawContext())
+      if (ctx->GetDrawContext()) {
          ctx->GetDrawContext()->BeginFrame(Draw::DebugFlags::NONE);
+      }
 
-      gpu->BeginHostFrame();
+      if (gpu)
+         gpu->BeginHostFrame();
 
-      coreState = CORE_RUNNING;
-      PSP_RunLoopUntil(UINT64_MAX);
+      PSP_RunLoopWhileState();
+      switch (coreState) {
+      case CORE_NEXTFRAME:
+      case CORE_POWERDOWN:
+         // Reached the end of the frame while running at full blast, all good. Set back to running for the next frame
+         coreState = CORE_RUNNING_CPU;
+         break;
+      default:
+         // We're not handling the various states used for debugging in the libretro port.
+         break;
+      }
 
-      gpu->EndHostFrame();
+      if (gpu)
+         gpu->EndHostFrame();
 
       if (ctx->GetDrawContext()) {
          ctx->GetDrawContext()->EndFrame();
@@ -1367,7 +1351,7 @@ namespace Libretro
 
    static void EmuThreadFunc()
    {
-      SetCurrentThreadName("Emu");
+      SetCurrentThreadName("EmuThread");
 
       for (;;)
       {
@@ -1375,15 +1359,15 @@ namespace Libretro
          {
             case EmuThreadState::START_REQUESTED:
                emuThreadState = EmuThreadState::RUNNING;
-               /* fallthrough */
+               [[fallthrough]];
             case EmuThreadState::RUNNING:
                EmuFrame();
                break;
             case EmuThreadState::PAUSE_REQUESTED:
                emuThreadState = EmuThreadState::PAUSED;
-               /* fallthrough */
+               [[fallthrough]];
             case EmuThreadState::PAUSED:
-               sleep_ms(1);
+               sleep_ms(1, "libretro-paused");
                break;
             default:
             case EmuThreadState::QUIT_REQUESTED:
@@ -1432,7 +1416,7 @@ namespace Libretro
       ctx->ThreadFrame(); // Eat 1 frame
 
       while (emuThreadState != EmuThreadState::PAUSED)
-         sleep_ms(1);
+         sleep_ms(1, "libretro-pause-poll");
    }
 
 } // namespace Libretro
@@ -1468,8 +1452,8 @@ bool retro_load_game(const struct retro_game_info *game)
 
    retro_check_backend();
 
-   coreState = CORE_POWERUP;
    ctx       = LibretroGraphicsContext::CreateGraphicsContext();
+
    INFO_LOG(Log::System, "Using %s backend", ctx->Ident());
 
    Core_SetGraphicsContext(ctx);
@@ -1478,14 +1462,14 @@ bool retro_load_game(const struct retro_game_info *game)
    useEmuThread              = ctx->GetGPUCore() == GPUCORE_GLES;
 
    // default to interpreter to allow startup in platforms w/o JIT capability
+   // TODO: I guess we should auto detect? And also, default to IR Interpreter...
    g_Config.iCpuCore         = (int)CPUCore::INTERPRETER;
 
    CoreParameter coreParam   = {};
    coreParam.enableSound     = true;
    coreParam.fileToStart     = Path(std::string(game->path));
-   coreParam.mountIso.clear();
    coreParam.startBreak      = false;
-   coreParam.headLess        = true;
+   coreParam.headLess        = true;  // really?
    coreParam.graphicsContext = ctx;
    coreParam.gpuCore         = ctx->GetGPUCore();
    check_variables(coreParam);
@@ -1499,32 +1483,37 @@ bool retro_load_game(const struct retro_game_info *game)
    // set cpuCore from libretro setting variable
    coreParam.cpuCore         =  (CPUCore)g_Config.iCpuCore;
 
-   std::string error_string;
-   if (!PSP_InitStart(coreParam, &error_string))
-   {
-      ERROR_LOG(Log::Boot, "%s", error_string.c_str());
-      return false;
-   }
+   g_pendingBoot = true;
 
    struct retro_core_option_display option_display;
 
    // Show/hide 'MSAA' and 'Texture Shader' options, Vulkan only
-   option_display.visible = (g_Config.iGPUBackend == (int)GPUBackend::VULKAN) ? true : false;
-#if 0 // see issue #16786
+   option_display.visible = (g_Config.iGPUBackend == (int)GPUBackend::VULKAN);
    option_display.key = "ppsspp_mulitsample_level";
    environ_cb(RETRO_ENVIRONMENT_SET_CORE_OPTIONS_DISPLAY, &option_display);
-#endif
    option_display.key = "ppsspp_texture_shader";
    environ_cb(RETRO_ENVIRONMENT_SET_CORE_OPTIONS_DISPLAY, &option_display);
 
    // Show/hide 'Buffered Frames' option, Vulkan/GL only
    option_display.visible = (g_Config.iGPUBackend == (int)GPUBackend::VULKAN ||
-         g_Config.iGPUBackend == (int)GPUBackend::OPENGL) ? true : false;
+      g_Config.iGPUBackend == (int)GPUBackend::OPENGL);
    option_display.key = "ppsspp_inflight_frames";
    environ_cb(RETRO_ENVIRONMENT_SET_CORE_OPTIONS_DISPLAY, &option_display);
 
    set_variable_visibility();
 
+   // NOTE: At this point we haven't really booted yet, but "in-game" we'll just keep polling
+   // PSP_InitUpdate until done.
+
+   // Launch the init process.
+   if (!PSP_InitStart(coreParam)) {
+      g_bootErrorString = coreParam.errorString;
+      // Can't really fail, the errors normally happen later during InitUpdate
+      ERROR_LOG(Log::Boot, "%s", g_bootErrorString.c_str());
+      g_pendingBoot = false;
+      return false;
+   }
+
    return true;
 }
 
@@ -1533,7 +1522,7 @@ void retro_unload_game(void)
 	if (Libretro::useEmuThread)
 		Libretro::EmuThreadStop();
 
-	PSP_Shutdown();
+	PSP_Shutdown(true);
 	g_VFS.Clear();
 
 	delete ctx;
@@ -1543,13 +1532,11 @@ void retro_unload_game(void)
 
 void retro_reset(void)
 {
-   std::string error_string;
+   PSP_Shutdown(true);
 
-   PSP_Shutdown();
-
-   if (!PSP_Init(PSP_CoreParameter(), &error_string))
+   if (BootState::Complete != PSP_Init(PSP_CoreParameter(), &g_bootErrorString))
    {
-      ERROR_LOG(Log::Boot, "%s", error_string.c_str());
+      ERROR_LOG(Log::Boot, "%s", g_bootErrorString.c_str());
       environ_cb(RETRO_ENVIRONMENT_SHUTDOWN, nullptr);
    }
 }
@@ -1610,19 +1597,22 @@ static void retro_input(void)
    float x_right = input_state_cb(0, RETRO_DEVICE_ANALOG, RETRO_DEVICE_INDEX_ANALOG_RIGHT, RETRO_DEVICE_ID_ANALOG_X) / 32767.0f;
    float y_right = input_state_cb(0, RETRO_DEVICE_ANALOG, RETRO_DEVICE_INDEX_ANALOG_RIGHT, RETRO_DEVICE_ID_ANALOG_Y) / -32767.0f;
 
-   __CtrlSetAnalogXY(CTRL_STICK_LEFT, x_left, y_left);
-   __CtrlSetAnalogXY(CTRL_STICK_RIGHT, x_right, y_right);
-
-   // Analog circle vs square gate compensation
-   // copied from ControlMapper.cpp's ConvertAnalogStick function
+   // Analog circle vs square gate compensation,
+   // deadzone and sensitivity copied from ControlMapper.cpp's
+   // ConvertAnalogStick and MapAxisValue functions
    const bool isCircular = g_Config.bAnalogIsCircular;
 
    float norm = std::max(fabsf(x_left), fabsf(y_left));
 
    if (norm == 0.0f)
+   {
+      __CtrlSetAnalogXY(CTRL_STICK_LEFT, x_left, y_left);
+      __CtrlSetAnalogXY(CTRL_STICK_RIGHT, x_right, y_right);
       return;
+   }
 
-   if (isCircular) {
+   if (isCircular)
+   {
       float newNorm = sqrtf(x_left * x_left + y_left * y_left);
       float factor = newNorm / norm;
       x_left *= factor;
@@ -1630,7 +1620,18 @@ static void retro_input(void)
       norm = newNorm;
    }
 
+   const float deadzone = g_Config.fAnalogDeadzone;
+   const float sensitivity = g_Config.fAnalogSensitivity;
+   const float sign = norm >= 0.0f ? 1.0f : -1.0f;
    float mappedNorm = norm;
+
+   // Apply deadzone
+   mappedNorm = Libretro::clamp((fabsf(mappedNorm) - deadzone) / (1.0f - deadzone), 0.0f, 1.0f);
+
+   // Apply sensitivity
+   if (mappedNorm != 0.0f)
+      mappedNorm = Libretro::clamp(mappedNorm * sensitivity * sign, -1.0f, 1.0f);
+
    x_left = Libretro::clamp(x_left / norm * mappedNorm, -1.0f, 1.0f);
    y_left = Libretro::clamp(y_left / norm * mappedNorm, -1.0f, 1.0f);
 
@@ -1640,29 +1641,45 @@ static void retro_input(void)
 
 void retro_run(void)
 {
-   if (PSP_IsIniting())
-   {
-      std::string error_string;
-      while (!PSP_InitUpdate(&error_string))
-         sleep_ms(4);
-
-      if (!PSP_IsInited())
-      {
-         ERROR_LOG(Log::Boot, "%s", error_string.c_str());
+   if (g_pendingBoot) {
+      BootState state = PSP_InitUpdate(&g_bootErrorString);
+      switch (state) {
+      case BootState::Failed:
+         g_pendingBoot = false;
+         ERROR_LOG(Log::Boot, "%s", g_bootErrorString.c_str());
          environ_cb(RETRO_ENVIRONMENT_SHUTDOWN, nullptr);
          return;
+      case BootState::Booting:
+         // Not done yet. Do maintenance stuff and bail.
+         retro_input();
+         ctx->SwapBuffers();
+         return;
+      case BootState::Off:
+         // shouldn't happen.
+         _dbg_assert_(false);
+         return;
       }
 
-      if (softwareRenderInitHack)
-      {
-         log_cb(RETRO_LOG_DEBUG, "Software rendering init hack for opengl triggered.\n");
-         softwareRenderInitHack = false;
-         g_Config.bSoftwareRendering = true;
-         retro_reset();
-      }
+      // BootState is BootState::Complete.
+      // Here's where we finish the boot process.
+      coreState = CORE_RUNNING_CPU;
+      g_bootErrorString.clear();
+      g_pendingBoot = false;
    }
 
-   check_variables(PSP_CoreParameter());
+   // TODO: This seems dubious.
+   if (softwareRenderInitHack)
+   {
+      log_cb(RETRO_LOG_DEBUG, "Software rendering init hack for opengl triggered.\n");
+      softwareRenderInitHack = false;
+      g_Config.bSoftwareRendering = true;
+      retro_reset();
+   }
+
+   bool updated;
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE_UPDATE, &updated)
+      && updated)
+      check_variables(PSP_CoreParameter());
 
    retro_input();
 
@@ -1736,7 +1753,7 @@ bool retro_serialize(void *data, size_t size)
    if (useEmuThread)
    {
       EmuThreadStart();
-      sleep_ms(4);
+      sleep_ms(4, "libretro-serialize");
    }
 
    return retVal;
@@ -1759,7 +1776,7 @@ bool retro_unserialize(const void *data, size_t size)
    if (useEmuThread)
    {
       EmuThreadStart();
-      sleep_ms(4);
+      sleep_ms(4, "libretro-unserialize");
    }
 
    return retVal;
@@ -1927,6 +1944,7 @@ void System_Notify(SystemNotification notification) {
 }
 bool System_MakeRequest(SystemRequestType type, int requestId, const std::string &param1, const std::string &param2, int64_t param3, int64_t param4) { return false; }
 void System_PostUIMessage(UIMessage message, const std::string &param) {}
+void System_RunOnMainThread(std::function<void()>) {}
 void NativeFrame(GraphicsContext *graphicsContext) {}
 void NativeResized() {}
 
@@ -1938,7 +1956,9 @@ inline int16_t Clamp16(int32_t sample) {
    return sample;
 }
 
-void System_AudioPushSamples(const int32_t *audio, int numSamples) {
+void System_AudioPushSamples(const int32_t *audio, int numSamples, float volume) {
+   // We ignore volume here, because it's handled by libretro presumably.
+
    // Convert to 16-bit audio for further processing.
    int16_t buffer[1024 * 2];
    int origSamples = numSamples * 2;
