@@ -48,11 +48,20 @@
 
 #include "Common/Data/Collections/TinySet.h"
 #include "Common/Data/Collections/FastVec.h"
+#include "Common/Data/Collections/CharQueue.h"
 #include "Common/Data/Convert/SmallDataConvert.h"
 #include "Common/Data/Text/Parsers.h"
 #include "Common/Data/Text/WrapText.h"
 #include "Common/Data/Encoding/Utf8.h"
+#include "Common/Buffer.h"
 #include "Common/File/Path.h"
+#include "Common/Math/SIMDHeaders.h"
+#include "Common/Math/CrossSIMD.h"
+// Get some more instructions for testing
+#if PPSSPP_ARCH(SSE2)
+#include <immintrin.h>
+#endif
+
 #include "Common/Input/InputState.h"
 #include "Common/Math/math_util.h"
 #include "Common/Render/DrawBuffer.h"
@@ -103,16 +112,15 @@ bool System_GetPropertyBool(SystemProperty prop) {
 }
 void System_Notify(SystemNotification notification) {}
 void System_PostUIMessage(UIMessage message, const std::string &param) {}
+void System_RunOnMainThread(std::function<void()>) {}
 void System_AudioGetDebugStats(char *buf, size_t bufSize) { if (buf) buf[0] = '\0'; }
 void System_AudioClear() {}
-void System_AudioPushSamples(const s32 *audio, int numSamples) {}
+void System_AudioPushSamples(const s32 *audio, int numSamples, float volume) {}
 
 // TODO: To avoid having to define these here, these should probably be turned into system "requests".
 // To clear the secret entirely, just save an empty string.
 bool NativeSaveSecret(std::string_view nameOfSecret, std::string_view data) { return false; }
-std::string NativeLoadSecret(std::string_view nameOfSecret) {
-	return "";
-}
+std::string NativeLoadSecret(std::string_view nameOfSecret) { return ""; }
 
 #if PPSSPP_PLATFORM(ANDROID)
 JNIEnv *getEnv() {
@@ -457,7 +465,7 @@ bool TestVFPUSinCos() {
 	return true;
 }
 
-bool TestMatrixTranspose() {
+bool TestVFPUMatrixTranspose() {
 	MatrixSize sz = M_4x4;
 	int matrix = 0;  // M000
 	u8 cols[4];
@@ -480,6 +488,7 @@ bool TestMatrixTranspose() {
 	return true;
 }
 
+// TODO: Hook this up again!
 void TestGetMatrix(int matrix, MatrixSize sz) {
 	INFO_LOG(Log::System, "Testing matrix %s", GetMatrixNotation(matrix, sz).c_str());
 	u8 fullMatrix[16];
@@ -987,14 +996,13 @@ bool TestIniFile() {
 	const std::string testLine2 = "# Just a comment";
 
 	std::string temp;
-	ParsedIniLine line;
-	line.ParseFrom(testLine);
+	ParsedIniLine line(testLine);
 	line.Reconstruct(&temp);
 	EXPECT_EQ_STR(testLine, temp);
 
 	temp.clear();
-	line.ParseFrom(testLine2);
-	line.Reconstruct(&temp);
+	ParsedIniLine line2(testLine2);
+	line2.Reconstruct(&temp);
 
 	EXPECT_EQ_STR(testLine2, temp);
 	return true;
@@ -1034,6 +1042,195 @@ bool TestColorConv() {
 	return true;
 }
 
+CharQueue GetQueue() {
+	CharQueue queue(5);
+	return queue;
+}
+
+bool TestCharQueue() {
+	// We use a tiny block size for testing.
+	CharQueue queue = GetQueue();
+
+	// Add 16 chars.
+	queue.push_back("abcdefghijkl");
+	queue.push_back("mnop");
+
+	std::string testStr;
+	queue.iterate_blocks([&](const char *buf, size_t sz) {
+		testStr.append(buf, sz);
+		return true;
+	});
+	EXPECT_EQ_STR(testStr, std::string("abcdefghijklmnop"));
+
+	EXPECT_EQ_CHAR(queue.peek(11), 'l');
+	EXPECT_EQ_CHAR(queue.peek(12), 'm');
+	EXPECT_EQ_CHAR(queue.peek(15), 'p');
+	EXPECT_EQ_INT(queue.block_count(), 3);  // Didn't fit in the first block, so the two pushes above should have each created one additional block.
+	EXPECT_EQ_INT(queue.size(), 16);
+	char dest[15];
+	EXPECT_EQ_INT(queue.pop_front_bulk(dest, 4), 4);
+	EXPECT_EQ_INT(queue.size(), 12);
+	EXPECT_EQ_MEM(dest, "abcd", 4);
+	EXPECT_EQ_INT(queue.pop_front_bulk(dest, 6), 6);
+	EXPECT_EQ_INT(queue.size(), 6);
+	EXPECT_EQ_MEM(dest, "efghij", 6);
+	queue.push_back("qr");
+	EXPECT_EQ_INT(queue.pop_front_bulk(dest, 4), 4);  // should pop off klmn
+	EXPECT_EQ_MEM(dest, "klmn", 4);
+	EXPECT_EQ_INT(queue.size(), 4);
+	EXPECT_EQ_CHAR(queue.peek(3), 'r');
+	queue.pop_front_bulk(dest, 4);
+	EXPECT_EQ_MEM(dest, "opqr", 4);
+	EXPECT_TRUE(queue.empty());
+	queue.push_back("asdf");
+	EXPECT_EQ_INT(queue.next_crlf_offset(), -1);
+	queue.push_back("\r\r\n");
+	EXPECT_EQ_INT(queue.next_crlf_offset(), 5);
+	return true;
+}
+
+bool TestBuffer() {
+	Buffer b = Buffer::Void();
+	b.Append("hello");
+	b.Append("world");
+	std::string temp;
+	b.Take(10, &temp);
+	EXPECT_EQ_STR(temp, std::string("helloworld"));
+	return true;
+}
+
+#if PPSSPP_ARCH(SSE2) && (defined(__GNUC__) || defined(__clang__) || defined(__INTEL_COMPILER))
+[[gnu::target("sse4.1")]]
+#endif
+bool TestSIMD() {
+#if PPSSPP_ARCH(SSE2)
+	__m128i x = _mm_set_epi16(0, 0x4444, 0, 0x3333, 0, 0x2222, 0, 0x1111);
+	__m128i y = _mm_packu_epi32_SSE2(x);
+
+	uint64_t testdata[2];
+	_mm_store_si128((__m128i *)testdata, y);
+	EXPECT_EQ_INT(testdata[0], 0x4444333322221111);
+	EXPECT_EQ_INT(testdata[1], 0);
+
+	__m128i a = _mm_set_epi16(0, 0x4444, 0, 0x3333, 0, 0x2222, 0, 0x1111);
+	__m128i b = _mm_set_epi16(0, (int16_t)0x8888, 0, 0x7777, 0, 0x6666, 0, 0x5555);
+	__m128i c = _mm_packu2_epi32_SSE2(a, b);
+	__m128i d = _mm_packu1_epi32_SSE2(b);
+
+	uint64_t testdata2[4];
+	_mm_store_si128((__m128i *)testdata2, c);
+	_mm_store_si128((__m128i *)testdata2 + 1, d);
+	EXPECT_EQ_INT(testdata2[0], 0x4444333322221111);
+	EXPECT_EQ_INT(testdata2[1], 0x8888777766665555);
+	EXPECT_EQ_INT(testdata2[2], 0x8888777766665555);
+	EXPECT_EQ_INT(testdata2[2], 0x8888777766665555);
+#endif
+
+	const int testval[2][4] = {
+		{ 0x1000, 0x2000, 0x3000, 0x7000 },
+		{ -0x1000, -0x2000, -0x3000, -0x7000 }
+	};
+
+	for (int i = 0; i < 2; i++) {
+		Vec4S32 s = Vec4S32::Load(testval[i]);
+		Vec4S32 square = s * s;
+		Vec4S32 square16 = s.Mul16(s);
+		EXPECT_EQ_INT(square[0], square16[0]);
+		EXPECT_EQ_INT(square[1], square16[1]);
+		EXPECT_EQ_INT(square[2], square16[2]);
+		EXPECT_EQ_INT(square[3], square16[3]);
+	}
+	return true;
+}
+
+static void PrintFloats(const float *f, int count) {
+	for (int i = 0; i < count; i++) {
+		printf("%.1ff, ", f[i]);
+	}
+	printf("\n");
+}
+
+static bool CompareFloats(const float *values, const float *known_good, int count, int line) {
+	int wrongCount = 0;
+
+	for (int i = 0; i < count; i++) {
+		if (values[i] != known_good[i]) {
+			wrongCount++;
+		}
+	}
+
+	if (wrongCount > 0) {
+		for (int i = 0; i < count; i++) {
+			bool wrong = values[i] != known_good[i];
+			printf("%d: %0.3f vs %0.3f %s\n", i + 1, values[i], known_good[i], wrong ? "!! MISMATCH" : "");
+		}
+		printf("At UnitTest.cpp:%d: %d / %d were wrong\n", line, wrongCount, count);
+		return false;
+	} else {
+		return true;
+	}
+}
+
+bool TestCrossSIMD() {
+	static const float a_values[16] = { 1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 7.0f, 8.0f, 9.0f, 10.0f, 11.0f, 12.0f, 13.0f, 14.0f, 15.0f };
+	static const float b_values[16] = { -12.0f, 3.0f, -2.5f, 5.0f, 31.0f, 0.5f, 4.0f, 6.0f, 7.0f, 13.0f, 12.0f, 51.0f, 81.0f, 32.0f };
+	static const float known_result[16] = { 395.0f, 171.0f, 41.5f, 170.0f, 942.0f, 410.5f, 111.5f, 475.0f, 1358.0f, 607.5f, 163.0f, 728.0f, 297.0f, 49.5f, 25.0f, 160.0f, };
+	float result[16];
+	Mat4F32 a(a_values);
+	Mat4F32 b(b_values);
+
+	Mul4x4By4x4(a, b).Store(result);
+	if (!CompareFloats(result, known_result, 16, __LINE__)) {
+		return false;
+	}
+
+	Mat4x3F32 d = Mat4x3F32(b_values + 2);
+	Mul4x3By4x4(d, a).Store(result);
+
+	static const float known_4x3_result[16] = { 332.5f, 371.0f, 404.5f, 438.0f, 80.5f, 95.0f, 105.5f, 116.0f, 192.0f, 237.0f, 269.0f, 301.0f, 790.0f, 1036.0f, 1185.0f, 1349.0f, };
+	if (!CompareFloats(result, known_4x3_result, 16, __LINE__)) {
+		return false;
+	}
+
+	static const float vec_values[4] = { 3.0f, 5.0f, 7.0f, 10000000.0f };
+	Vec4F32 v = Vec4F32::Load(vec_values);
+
+	v.AsVec3ByMatrix44(b).Store3(result);
+
+	static const float known_vec_result[3] = { 249.0f, 134.5f, 96.5f, };
+	if (!CompareFloats(result, known_vec_result, ARRAY_SIZE(known_vec_result), __LINE__)) {
+		return false;
+	}
+	Vec4F32 scale = Vec4F32::Load(a_values);
+	Vec4F32 translate = Vec4F32::Load(b_values);
+
+	TranslateAndScaleInplace(a, scale, translate);
+	a.Store(result);
+
+	static const float known_scale_result[16] = { -47.0f, 16.0f, -1.0f, 36.0f, -103.0f, 41.0f, 1.5f, 81.0f, -146.0f, 61.0f, 3.5f, 117.0f, 14.0f, 30.0f, 0.0f, 0.0f,};
+	if (!CompareFloats(result, known_scale_result, ARRAY_SIZE(known_scale_result), __LINE__)) {
+		return false;
+	}
+
+	// PrintFloats(result, 16);
+
+	return true;
+}
+
+bool TestVolumeFunc() {
+	for (int i = 0; i <= 20; i++) {
+		float mul = Volume10ToMultiplier(i);
+
+		int vol100 = MultiplierToVolume100(mul);
+		float mul2 = Volume100ToMultiplier(vol100);
+
+		bool smaller = (fabsf(mul2 - mul) < 0.02f);
+		EXPECT_TRUE(smaller);
+		// printf("%d -> %f -> %d -> %f\n", i, mul, vol100, mul2);
+	}
+	return true;
+}
+
 typedef bool (*TestFunc)();
 struct TestItem {
 	const char *name;
@@ -1073,7 +1270,7 @@ TestItem availableTests[] = {
 	TEST_ITEM(Parsers),
 	TEST_ITEM(IRPassSimplify),
 	TEST_ITEM(Jit),
-	TEST_ITEM(MatrixTranspose),
+	TEST_ITEM(VFPUMatrixTranspose),
 	TEST_ITEM(ParseLBN),
 	TEST_ITEM(QuickTexHash),
 	TEST_ITEM(CLZ),
@@ -1094,6 +1291,11 @@ TestItem availableTests[] = {
 	TEST_ITEM(Substitutions),
 	TEST_ITEM(IniFile),
 	TEST_ITEM(ColorConv),
+	TEST_ITEM(CharQueue),
+	TEST_ITEM(Buffer),
+	TEST_ITEM(SIMD),
+	TEST_ITEM(CrossSIMD),
+	TEST_ITEM(VolumeFunc),
 };
 
 int main(int argc, const char *argv[]) {
